from typing import Optional

from abqpy.decorators import abaqus_class_doc
from .OdbInstance import OdbInstance
from .OdbPart import OdbPart
from .SectionPoint import SectionPoint
from ..UtilityAndView.abaqusConstants import DEFORMABLE_BODY, SymbolicConstant, THREE_D


@abaqus_class_doc
class FieldBulkData:
    """The FieldBulkData object represents the entire field data for a class of elements or
    nodes. All elements in a class correspond to the same element type and material.

    .. note:: 
        This object can be accessed by::

            import odbAccess
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].bulkDataBlocks[i]
    """

    #: A SymbolicConstant specifying the position of the output in the element. Possible values
    #: are:
    #: 
    #: - NODAL, specifying the values calculated at the nodes.
    #: - INTEGRATION_POINT, specifying the values calculated at the integration points.
    #: - ELEMENT_NODAL, specifying the values obtained by extrapolating results calculated at
    #:   the integration points.
    #: - ELEMENT_FACE.
    #: - CENTROID, specifying the value at the centroid obtained by extrapolating results
    #:   calculated at the integration points.
    position: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the output type. Possible values are SCALAR, VECTOR,
    #: TENSOR_3D_FULL, TENSOR_3D_PLANAR, TENSOR_3D_SURFACE, TENSOR_2D_PLANAR, and
    #: TENSOR_2D_SURFACE.
    type: Optional[SymbolicConstant] = None

    #: An :py:class:`~abaqus.Odb.OdbInstance.OdbInstance` object specifying the part to which the labels belong.
    instance: OdbInstance = OdbInstance(
        "instance", OdbPart("part", THREE_D, DEFORMABLE_BODY)
    )

    #: A :py:class:`~abaqus.Odb.SectionPoint.SectionPoint` object specifying the section point number of the current block of data.
    sectionPoint: Optional[SectionPoint] = None

    #: A sequence of Ints specifying the element labels of the elements in the block.
    #: **elementLabels** is valid only if **position** = INTEGRATION_POINT, CENTROID, ELEMENT_NODAL,
    #: or ELEMENT_FACE.
    elementLabels: tuple = ()

    #: A sequence of Ints specifying the node labels of the nodes in the block. **nodelabels** is
    #: valid only if **position** = ELEMENT_NODAL or NODAL.
    nodeLabels: tuple = ()

    #: A sequence of Strings specifying the component labels.
    componentLabels: tuple = ()

    #: A sequence of Ints specifying the integration points in the elements in the block.
    #: **integrationPoints** is available only if **position** = INTEGRATION_POINT.
    integrationPoints: tuple = ()

    #: A tuple of Floats specifying data in the form described by **type**. If **type** = TENSOR or
    #: VECTOR, **data** is a sequence containing the components for each element or node in the
    #: block. If the underlying data are in double precision, an exception will be thrown.
    data: tuple = ()

    #: A tuple of Floats specifying data in the form described by **type**. If **type** = TENSOR or
    #: VECTOR, **conjugateData** is a sequence containing the imaginary part of the components
    #: for each element or node in the block. If the underlying data are in double precision,
    #: an exception will be thrown.
    conjugateData: tuple = ()

    #: A sequence of Floats specifying the calculated von Mises stress at each output location
    #: in the block of element data, or NULL. The value is valid only when the
    #: **validInvariants** member includes MISES; otherwise, the value is indeterminate.
    #: Conjugate data will be ignored in invariant calculation.
    mises: tuple = ()

    #: A pointer to an array of Floats specifying the quaternion representing the local
    #: coordinate system (the rotation from global to local) at each output location. The
    #: quaternion is returned in the form q=(q,q0), which is the reverse of that shown in
    #: [Rotation
    #: variables](https://help.3ds.com/2022/english/DSSIMULIA_Established/SIMACAETHERefMap/simathe-c-rotationvars.htm?ContextScope=all).
    #: **localCoordSystem** is available for TENSOR data written in a local coordinate system. It
    #: is also available for VECTOR data for connector element outputs. For connector element
    #: outputs the quaternion form is q=(q0,q)q=(q0,q), which represents the rotation from
    #: local to global. If the underlying data are in double precision, an exception will be
    #: thrown.
    localCoordSystem: Optional[float] = None



from typing import Optional

from abqpy.decorators import abaqus_class_doc
from .InteractionState import InteractionState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class ConcentratedRadiationToAmbientState(InteractionState):
    """The ConcentratedRadiationToAmbientState object stores the propagating data for a
    ConcentratedRadiationToAmbient object. One instance of this object is created internally
    by the ConcentratedRadiationToAmbient object for each step. The instance is also deleted
    internally by the ConcentratedRadiationToAmbient object.
    The ConcentratedRadiationToAmbientState object has no constructor or methods.
    The ConcentratedRadiationToAmbientState object is derived from the InteractionState
    object.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]

        The corresponding analysis keywords are:

        - CRADIATE
    """

    #: A Float specifying the ambient temperature.
    ambientTemperature: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the **ambientTemperature** member.
    #: Possible values are UNSET, SET, UNCHANGED, and FREED.
    ambientTemperatureState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **ambientTemperatureAmp**
    #: member. Possible values are UNSET, SET, UNCHANGED, and FREED.
    ambientTemperatureAmpState: Optional[SymbolicConstant] = None

    #: A Float specifying the emissivity.
    emissivity: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the **emissivity** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    emissivityState: Optional[SymbolicConstant] = None

    #: A Float specifying the area associated with the node where the concentrated radiation is
    #: applied.
    nodalArea: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the **nodalArea** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    nodalAreaState: Optional[SymbolicConstant] = None

    #: A String specifying the name of the Amplitude object that gives the variation of the
    #: ambient temperature with time.
    ambientTemperatureAmp: str = ""

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ConstrainedSketch import ConstrainedSketch
from ..Model.ModelBase import ModelBase


@abaqus_class_doc
class SketchModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note:: 
        This object can be accessed by::

            mdb.models[name]
    """

    @abaqus_method_doc
    def ConstrainedSketch(
        self,
        name: str,
        sheetSize: float,
        gridSpacing: Optional[float] = None,
        transform: tuple = (),
    ) -> ConstrainedSketch:
        """This method creates a ConstrainedSketch object. If the sketch cannot be created, the
        method returns None.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].ConstrainedSketch

        Parameters
        ----------
        name
            A String specifying the repository key.
        sheetSize
            A Float specifying the sheet size.
        gridSpacing
            A Float specifying the spacing between gridlines. Possible values are Floats >> 0. The
            default value is approximately 2 percent of **sheetSize**.
        transform
            A sequence of sequences of Floats specifying the three-dimensional orientation of the
            sketch. The sequence is a 3 × 4 transformation matrix specifying the axis of rotation
            and the translation vector. Possible values are any Floats.The default value for the
            axis of rotation is the identity matrix`(1.0, 0.0, 0.0),  (0.0, 1.0, 0.0),  (0.0, 0.0,
            1.0)`The default value for the translation vector is`(0.0, 0.0, 0.0)`The default values
            position the sketch on the **X - Y** plane centered at the origin.

        Returns
        -------
        sketch: ConstrainedSketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object.
        """
        self.sketches[name] = sketch = ConstrainedSketch(
            name, sheetSize, gridSpacing, transform
        )
        return sketch



from typing import List

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class Profile:
    """The Profile object defines the geometrical properties of a beam cross-section. Profile
    is an abstract base type.

    .. note:: 
        This object can be accessed by::

            import section
            mdb.models[name].profiles[name]
            import odbSection
            session.odbs[name].profiles[name]
    """

    #: A String specifying the repository key.
    name: str = ""

    @abaqus_method_doc
    def beamProfilesFromOdb(self, fileName: str):
        """This method creates Profile objects by reading an output database. The new profiles are
        placed in the profiles repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].beamProfilesFromOdb

        Parameters
        ----------
        fileName
            A String specifying the name of the output database file (including the .odb extension)
            to be read. The String can also be the full path to the output database file if it is
            located in another directory.

        Returns
        -------
        List[Profile]
            A list of Profile objects.

        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc
from .InteractionState import InteractionState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class IncidentWaveState(InteractionState):
    """The IncidentWaveState object stores the propagating data of an IncidentWave object in a
    step. One instance of this object is created internally by the IncidentWave object for
    each step. The instance is also deleted internally by the IncidentWave object.
    The IncidentWaveState object has no constructor or methods.
    The IncidentWaveState object is derived from the InteractionState object.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]

        The corresponding analysis keywords are:

        - INCIDENT WAVE INTERACTION
    """

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values
    #: are:NOT_YET_ACTIVECREATEDPROPAGATEDMODIFIEDDEACTIVATEDNO_LONGER_ACTIVETYPE_NOT_APPLICABLEINSTANCE_NOT_APPLICABLEBUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None



from typing import Optional, Tuple

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Datum import Datum
from .DatumAxis import DatumAxis
from .DatumPoint import DatumPoint
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class DatumCsys(Datum):
    """The DatumCsys object has no direct constructor; it is created when a Feature object is
    created. For example, the DatumCsysByOffset method creates a Feature object that creates
    a DatumCsys object.
    The DatumCsys object is derived from the Datum object.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].datums[i]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].datums[i]
            mdb.models[name].rootAssembly.connectorOrientations[i].localCsys1
            mdb.models[name].rootAssembly.connectorOrientations[i].localCsys2
            mdb.models[name].rootAssembly.datums[i]
            mdb.models[name].rootAssembly.instances[name].datums[i]
            import odbAccess
            session.odbs[name].rootAssembly.connectorOrientations[i].localCsys1
            session.odbs[name].rootAssembly.connectorOrientations[i].localCsys2
    """

    #: A SymbolicConstant specifying the type of the coordinate system. Possible values are
    #: CARTESIAN, CYLINDRICAL, and SPHERICAL.
    coordSysType: Optional[SymbolicConstant] = None

    #: A :py:class:`~abaqus.Datum.DatumPoint.DatumPoint` object specifying the origin of the coordinate system.
    origin: DatumPoint = DatumPoint()

    #: A :py:class:`~abaqus.Datum.DatumAxis.DatumAxis` object specifying the 1-direction of the coordinate system.
    axis1: DatumAxis = DatumAxis()

    #: A :py:class:`~abaqus.Datum.DatumAxis.DatumAxis` object specifying the 2-direction of the coordinate system.
    axis2: DatumAxis = DatumAxis()

    #: A :py:class:`~abaqus.Datum.DatumAxis.DatumAxis` object specifying the 3-direction of the coordinate system.
    axis3: DatumAxis = DatumAxis()

    @abaqus_method_doc
    def globalToLocal(
        self, coordinates: Tuple[float, float, float]
    ) -> Tuple[float, float, float]:
        """This method transforms specified coordinates in the global coordinate system into this
        local coordinate system.

        .. versionadded:: 2022
            The `globalToLocal` method was added.

        Parameters
        ----------
        coordinates
            A tuple of three Floats representing the coordinates in the global coordinate system.

        Returns
        -------
        Tuple[float, float, float]
            A tuple of three Floats representing the coordinates in this local coordinate system.
        """
        ...

    @abaqus_method_doc
    def localToGlobal(
        self, coordinates: Tuple[float, float, float]
    ) -> Tuple[float, float, float]:
        """This method transforms specified coordinates in this local coordinate system into the global coordinate system.

        .. versionadded:: 2022
            The `localToGlobal` method was added.

        Parameters
        ----------
        coordinates
            A tuple of three Floats representing the coordinates in the local coordinate system.

        Returns
        -------
        Tuple[float, float, float]
            A tuple of three Floats representing the coordinates in this global coordinate system.
        """
        ...



from typing import Union, Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Load import Load
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (AXIS_1, Boolean, OFF, ON, SHEAR, SymbolicConstant,
                                              UNIFORM, UNSET)


@abaqus_class_doc
class SurfaceTraction(Load):
    """The SurfaceTraction object defines surface traction on a region.
    The SurfaceTraction object is derived from the Load object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].loads[name]
    """

    #: A String specifying the load repository key.
    name: str = ""

    #: A Float specifying an additional rotation of **directionVector** about an axis. The
    #: default value is 0.0.
    angle: float = 0

    #: A SymbolicConstant specifying the axis about which to apply an additional rotation of
    #: **directionVector**. Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is
    #: AXIS_1.
    axis: SymbolicConstant = AXIS_1

    #: A Boolean specifying whether the direction of the force changes with rotation. The
    #: default value is ON.This parameter may be modified only if **traction** is GENERAL. You
    #: should provide the **follower** argument only if it is valid for the specified step.
    follower: Boolean = ON

    #: A Boolean specifying whether the to maintain a constant resultant force by defining
    #: traction per unit undeformed area. If **resultant** is OFF, traction is defined per unit
    #: deformed area. The default value is OFF.You should provide the **resultant** argument only
    #: if it is valid for the specified step.
    resultant: Boolean = OFF

    #: A SymbolicConstant specifying how to apply surface traction. Possible values are SHEAR
    #: and GENERAL. The default value is SHEAR.
    traction: SymbolicConstant = SHEAR

    #: A SymbolicConstant specifying how the surface traction is distributed spatially.
    #: Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A String specifying the name of the AnalyticalField object associated with this load.
    #: The **field** argument applies only when **distributionType** = FIELD. The default value is an
    #: empty string.
    field: str = ""

    #: A String specifying a CSYS defined by a user-subroutine. If **userCsys** = None, the degrees
    #: of freedom are defined in the global coordinate system or by the **localCsys** parameter
    #: if defined. The default value is "None".
    userCsys: str = ""

    #: None or a DatumCsys object specifying the local coordinate system of the load's degrees
    #: of freedom. If **localCsys** = None, the degrees of freedom are defined in the global
    #: coordinate system or by the **userCsys** parameter if defined. When this member is
    #: queried, it returns an Int. The default value is None.
    localCsys: Optional[int] = None

    #: A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object of length 2 specifying the direction of the load. Instead of
    #: through a ConstrainedSketchVertex, each point may be specified through a tuple of coordinates. If
    #: **traction** is SHEAR, then **directionVector** will be projected onto the region surface.
    #: This parameter is available only if **traction** is GENERAL or SHEAR.
    directionVector: tuple = ()

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the load is applied.
    region: Region = Region()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        magnitude: float,
        distributionType: SymbolicConstant = UNIFORM,
        field: str = "",
        amplitude: str = UNSET,
        angle: float = 0,
        axis: SymbolicConstant = AXIS_1,
        localCsys: Optional[int] = None,
        userCsys: str = "",
        directionVector: tuple = (),
        follower: Boolean = ON,
        resultant: Boolean = OFF,
        traction: SymbolicConstant = SHEAR,
    ):
        """This method creates a SurfaceTraction object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].SurfaceTraction

        Parameters
        ----------
        name
            A String specifying the load repository key.
        createStepName
            A String specifying the name of the step in which the load is created.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the load is applied.
        magnitude
            A Float or Complex specifying the load magnitude. **magnitude** is optional if
            **distributionType** = USER_DEFINED.
        distributionType
            A SymbolicConstant specifying how the surface traction is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        angle
            A Float specifying an additional rotation of **directionVector** about an axis. The
            default value is 0.0.
        axis
            A SymbolicConstant specifying the axis about which to apply an additional rotation of
            **directionVector**. Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is
            AXIS_1.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the load's degrees
            of freedom. If **localCsys** = None, the degrees of freedom are defined in the global
            coordinate system or by the **userCsys** parameter if defined. When this member is
            queried, it returns an Int. The default value is None.
        userCsys
            A String specifying a CSYS defined by a user-subroutine. If **userCsys** = None, the degrees
            of freedom are defined in the global coordinate system or by the **localCsys** parameter
            if defined. The default value is "None".
        directionVector
            A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object of length 2 specifying the direction of the load. Instead of
            through a ConstrainedSketchVertex, each point may be specified through a tuple of coordinates. If
            **traction** is SHEAR, then **directionVector** will be projected onto the region surface.
            This parameter is available only if **traction** is GENERAL or SHEAR.
        follower
            A Boolean specifying whether the direction of the force changes with rotation. The
            default value is ON.This parameter may be modified only if **traction** is GENERAL. You
            should provide the **follower** argument only if it is valid for the specified step.
        resultant
            A Boolean specifying whether the to maintain a constant resultant force by defining
            traction per unit undeformed area. If **resultant** is OFF, traction is defined per unit
            deformed area. The default value is OFF.You should provide the **resultant** argument only
            if it is valid for the specified step.
        traction
            A SymbolicConstant specifying how to apply surface traction. Possible values are SHEAR
            and GENERAL. The default value is SHEAR.

        Returns
        -------
        SurfaceTraction
            A :py:class:`~abaqus.Load.SurfaceTraction.SurfaceTraction` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        distributionType: SymbolicConstant = UNIFORM,
        field: str = "",
        amplitude: str = UNSET,
        angle: float = 0,
        axis: SymbolicConstant = AXIS_1,
        localCsys: Optional[int] = None,
        userCsys: str = "",
        directionVector: tuple = (),
        follower: Boolean = ON,
        resultant: Boolean = OFF,
        traction: SymbolicConstant = SHEAR,
    ):
        """This method modifies the data for an existing SurfaceTraction object in the step where
        it is created.

        Parameters
        ----------
        distributionType
            A SymbolicConstant specifying how the surface traction is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        field
            A String specifying the name of the AnalyticalField object associated with this load.
            The **field** argument applies only when **distributionType** = FIELD. The default value is an
            empty string.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        angle
            A Float specifying an additional rotation of **directionVector** about an axis. The
            default value is 0.0.
        axis
            A SymbolicConstant specifying the axis about which to apply an additional rotation of
            **directionVector**. Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is
            AXIS_1.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the load's degrees
            of freedom. If **localCsys** = None, the degrees of freedom are defined in the global
            coordinate system or by the **userCsys** parameter if defined. When this member is
            queried, it returns an Int. The default value is None.
        userCsys
            A String specifying a CSYS defined by a user-subroutine. If **userCsys** = None, the degrees
            of freedom are defined in the global coordinate system or by the **localCsys** parameter
            if defined. The default value is "None".
        directionVector
            A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object of length 2 specifying the direction of the load. Instead of
            through a ConstrainedSketchVertex, each point may be specified through a tuple of coordinates. If
            **traction** is SHEAR, then **directionVector** will be projected onto the region surface.
            This parameter is available only if **traction** is GENERAL or SHEAR.
        follower
            A Boolean specifying whether the direction of the force changes with rotation. The
            default value is ON.This parameter may be modified only if **traction** is GENERAL. You
            should provide the **follower** argument only if it is valid for the specified step.
        resultant
            A Boolean specifying whether the to maintain a constant resultant force by defining
            traction per unit undeformed area. If **resultant** is OFF, traction is defined per unit
            deformed area. The default value is OFF.You should provide the **resultant** argument only
            if it is valid for the specified step.
        traction
            A SymbolicConstant specifying how to apply surface traction. Possible values are SHEAR
            and GENERAL. The default value is SHEAR.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        magnitude: Union[SymbolicConstant, float] = None,
        amplitude: str = "",
    ):
        """This method modifies the propagating data for an existing SurfaceTraction object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the load is modified.
        magnitude
            A Float, a Complex, or the SymbolicConstant UNCHANGED specifying the load magnitude.
            UNCHANGED should be used if the magnitude is propagated from the previous analysis step.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous analysis step. FREED should be used if the
            load has no amplitude reference. You should provide the **amplitude** argument only if it
            is valid for the specified step.
        """
        ...



from typing import Optional, Tuple

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Leaf import Leaf
from ..Datum.Datum import Datum
from ..UtilityAndView.abaqusConstants import DEFAULT_MODEL, SymbolicConstant


@abaqus_class_doc
class LeafFromDatums(Leaf):
    """The LeafFromDatums object can be used whenever a Leaf object is expected as an argument.
    Leaf objects are used to specify the items in a display group. Leaf objects are
    constructed as temporary objects, which are then used as arguments to DisplayGroup
    commands.
    The LeafFromDatums object is derived from the Leaf object.

    .. note:: 
        This object can be accessed by::

            import displayGroupMdbToolset
    """

    #: A SymbolicConstant specifying the leaf type. Possible values are EMPTY_LEAF,
    #: DEFAULT_MODEL, ALL_ELEMENTS, ALL_NODES, and ALL_SURFACES.
    leafType: Optional[SymbolicConstant] = None

    @abaqus_method_doc
    def __init__(self, datumSeq: Tuple[Datum, ...]):
        """This method creates a Leaf object from a sequence of datum objects. Leaf objects specify
        the items in a display group.

        .. note:: 
            This function can be accessed by::

                LeafFromDatums

        Parameters
        ----------
        datumSeq
            A sequence of datum objects.

        Returns
        -------
        LeafFromDatums
            A :py:class:`~abaqus.DisplayGroup.LeafFromDatums.LeafFromDatums` object.
        """
        super().__init__(DEFAULT_MODEL)



from typing import Union, Optional, Tuple

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Annotation import Annotation
from ..UtilityAndView.abaqusConstants import (BOTTOM_LEFT, Boolean, JUSTIFY_LEFT, OFF,
                                              SymbolicConstant, TRANSPARENT)
from .._OptionsBase import _OptionsBase


@abaqus_class_doc
class Text(Annotation, _OptionsBase):
    """The Text object stores the text settings and location of a text annotation.
    The Text object is derived from the Annotation object.

    .. note:: 
        This object can be accessed by::

            import annotationToolset
            mdb.annotations[name]
            session.odbs[name].userData.annotations[name]
            session.viewports[name].annotationsToPlot[i]
    """

    #: A Float specifying the width in millimeters of the Text object.
    width: Optional[float] = None

    #: A Float specifying the height in millimeters of the Text object.
    height: Optional[float] = None

    #: A String specifying the annotation repository key.
    name: str

    #: A String specifying the text of the Text object. The default value is an empty string.
    text: str = ""

    #: A pair of Floats specifying the **X**- and **Y**-offsets in millimeters of the Text object
    #: from **anchor**. The default value is (0, 0).
    offset: Tuple[float, ...] = (0.0, 0.0)

    #: A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
    #: specifies the **X**- and **Y** coordinates as percentages of the viewport width and height.
    #: A Sequence of three Floats specifies the **X**-, **Y**-, and **Z**-coordinates of a point in
    #: the model coordinate system. A SymbolicConstant specifies a relative position. Possible
    #: values are:
    #: 
    #: - BOTTOM_LEFT
    #: - BOTTOM_CENTER
    #: - BOTTOM_RIGHT
    #: - CENTER_LEFT
    #: - CENTER
    #: - CENTER_RIGHT
    #: - TOP_LEFT
    #: - TOP_CENTER
    #: - TOP_RIGHT
    #: 
    #: The default value is BOTTOM_LEFT.
    anchor: Union[SymbolicConstant, float] = BOTTOM_LEFT

    #: A SymbolicConstant or a sequence of Floats specifying a point. The sequence of two
    #: Floats specifies the **X**- and **Y**-coordinates of the reference point of the Text
    #: annotation given as percentages of its width and height. The SymbolicConstant indicates
    #: a relative position. Possible values are:
    #: 
    #: - BOTTOM_LEFT
    #: - BOTTOM_CENTER
    #: - BOTTOM_RIGHT
    #: - CENTER_LEFT
    #: - CENTER
    #: - CENTER_RIGHT
    #: - TOP_LEFT
    #: - TOP_CENTER
    #: - TOP_RIGHT
    #: 
    #: The default value is BOTTOM_LEFT.
    referencePoint: Union[SymbolicConstant, float] = BOTTOM_LEFT

    #: A Float specifying the amount of rotation in degrees about **referencePoint**. The default
    #: value is 0.0.
    rotationAngle: float = 0.0

    #: A String specifying the color of the Text object. Possible string values are any valid
    #: color. The default value is "White".
    color: str = "White"

    #: A String specifying the font of the Text object. Possible string values are any valid
    #: font specification. The default value is "-*-verdana-medium-r-normal--120-*".
    font: str = "-*-verdana-medium-r-normal--120-*"

    #: A SymbolicConstant specifying the Text object background style. Possible values are
    #: MATCH, TRANSPARENT, and OTHER. The default value is TRANSPARENT.
    backgroundStyle: SymbolicConstant = TRANSPARENT

    #: A String specifying the color of the Text object background. Possible string values are
    #: any valid color. The default value matches the viewport background.
    backgroundColor: str = ""

    #: A Boolean specifying whether the box around the text is shown. The default value is OFF.
    box: Boolean = OFF

    #: A SymbolicConstant specifying the Text object justification for multiline text. Possible
    #: values are JUSTIFY_LEFT, JUSTIFY_CENTER, and JUSTIFY_RIGHT. The default value is
    #: JUSTIFY_LEFT.
    justification: SymbolicConstant = JUSTIFY_LEFT

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        text: str = "",
        offset: Tuple[float, ...] = (0.0, 0.0),
        anchor: Union[SymbolicConstant, float] = BOTTOM_LEFT,
        referencePoint: Union[SymbolicConstant, float] = BOTTOM_LEFT,
        rotationAngle: float = 0.0,
        color: str = "White",
        font: str = "-*-verdana-medium-r-normal--120-*",
        backgroundStyle: SymbolicConstant = TRANSPARENT,
        backgroundColor: str = "",
        box: Boolean = OFF,
        justification: SymbolicConstant = JUSTIFY_LEFT,
    ):
        """This method creates a Text object.

        .. note:: 
            This function can be accessed by::

                mdb.Text
                session.odbs[name].userData.Text

        Parameters
        ----------
        name
            A String specifying the annotation repository key.
        text
            A String specifying the text of the Text object. The default value is an empty string.
        offset
            A pair of Floats specifying the **X**- and **Y**-offsets in millimeters of the Text object
            from **anchor**. The default value is (0, 0).
        anchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**- and **Y** coordinates as percentages of the viewport width and height.
            A Sequence of three Floats specifies the **X**-, **Y**-, and **Z**-coordinates of a point in
            the model coordinate system. A SymbolicConstant specifies a relative position. Possible
            values are:
            
            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT
                
            The default value is BOTTOM_LEFT.
        referencePoint
            A SymbolicConstant or a sequence of Floats specifying a point. The sequence of two
            Floats specifies the **X**- and **Y**-coordinates of the reference point of the Text
            annotation given as percentages of its width and height. The SymbolicConstant indicates
            a relative position. Possible values are:
            
            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT
                
            The default value is BOTTOM_LEFT.
        rotationAngle
            A Float specifying the amount of rotation in degrees about **referencePoint**. The default
            value is 0.0.
        color
            A String specifying the color of the Text object. Possible string values are any valid
            color. The default value is "White".
        font
            A String specifying the font of the Text object. Possible string values are any valid
            font specification. The default value is "-*-verdana-medium-r-normal--120-*".
        backgroundStyle
            A SymbolicConstant specifying the Text object background style. Possible values are
            MATCH, TRANSPARENT, and OTHER. The default value is TRANSPARENT.
        backgroundColor
            A String specifying the color of the Text object background. Possible string values are
            any valid color. The default value matches the viewport background.
        box
            A Boolean specifying whether the box around the text is shown. The default value is OFF.
        justification
            A SymbolicConstant specifying the Text object justification for multiline text. Possible
            values are JUSTIFY_LEFT, JUSTIFY_CENTER, and JUSTIFY_RIGHT. The default value is
            JUSTIFY_LEFT.

        Returns
        -------
        Text
            A :py:class:`~abaqus.Annotation.Text.Text` object.

        """
        super().__init__()
        self.name = name
        self.text = text
        self.offset = offset
        self.anchor = anchor
        self.referencePoint = referencePoint
        self.rotationAngle = rotationAngle
        self.color = color
        self.font = font
        self.backgroundStyle = backgroundStyle
        self.backgroundColor = backgroundColor
        self.box = box
        self.justification = justification

    @abaqus_method_doc
    def setValues(
        self,
        text: str = "",
        offset: Tuple[float, ...] = (0.0, 0.0),
        anchor: Union[SymbolicConstant, float] = BOTTOM_LEFT,
        referencePoint: Union[SymbolicConstant, float] = BOTTOM_LEFT,
        rotationAngle: float = 0.0,
        color: str = "White",
        font: str = "-*-verdana-medium-r-normal--120-*",
        backgroundStyle: SymbolicConstant = TRANSPARENT,
        backgroundColor: str = "",
        box: Boolean = OFF,
        justification: SymbolicConstant = JUSTIFY_LEFT,
    ):
        """This method modifies the Text object.

        Parameters
        ----------
        text
            A String specifying the text of the Text object. The default value is an empty string.
        offset
            A pair of Floats specifying the **X**- and **Y**-offsets in millimeters of the Text object
            from **anchor**. The default value is (0, 0).
        anchor
            A SymbolicConstant or a sequence of Floats specifying a point. A sequence of two Floats
            specifies the **X**- and **Y** coordinates as percentages of the viewport width and height.
            A Sequence of three Floats specifies the **X**-, **Y**-, and **Z**-coordinates of a point in
            the model coordinate system. A SymbolicConstant specifies a relative position. Possible
            values
            are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is BOTTOM_LEFT.
        referencePoint
            A SymbolicConstant or a sequence of Floats specifying a point. The sequence of two
            Floats specifies the **X**- and **Y**-coordinates of the reference point of the Text
            annotation given as percentages of its width and height. The SymbolicConstant indicates
            a relative position. Possible values
            are:

            - BOTTOM_LEFT
            - BOTTOM_CENTER
            - BOTTOM_RIGHT
            - CENTER_LEFT
            - CENTER
            - CENTER_RIGHT
            - TOP_LEFT
            - TOP_CENTER
            - TOP_RIGHT

            The default value is BOTTOM_LEFT.
        rotationAngle
            A Float specifying the amount of rotation in degrees about **referencePoint**. The default
            value is 0.0.
        color
            A String specifying the color of the Text object. Possible string values are any valid
            color. The default value is "White".
        font
            A String specifying the font of the Text object. Possible string values are any valid
            font specification. The default value is "-*-verdana-medium-r-normal--120-*".
        backgroundStyle
            A SymbolicConstant specifying the Text object background style. Possible values are
            MATCH, TRANSPARENT, and OTHER. The default value is TRANSPARENT.
        backgroundColor
            A String specifying the color of the Text object background. Possible string values are
            any valid color. The default value matches the viewport background.
        box
            A Boolean specifying whether the box around the text is shown. The default value is OFF.
        justification
            A SymbolicConstant specifying the Text object justification for multiline text. Possible
            values are JUSTIFY_LEFT, JUSTIFY_CENTER, and JUSTIFY_RIGHT. The default value is
            JUSTIFY_LEFT.

        """
        super().setValues(
            text=text,
            offset=offset,
            anchor=anchor,
            referencePoint=referencePoint,
            rotationAngle=rotationAngle,
            color=color,
            font=font,
            backgroundStyle=backgroundStyle,
            backgroundColor=backgroundColor,
            box=box,
            justification=justification,
        )



from typing import Union

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Path import Path
from .Spectrum import Spectrum
from .Stream import Stream
from ..Session.SessionBase import SessionBase
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class PathSession(SessionBase):
    
    @abaqus_method_doc
    def Path(
        self,
        name: str,
        type: SymbolicConstant,
        expression: tuple,
        circleDefinition: SymbolicConstant,
        numSegments: int,
        startAngle: float,
        endAngle: float,
        radius: Union[SymbolicConstant, float],
        radialAngle: float,
        startRadius: Union[SymbolicConstant, float],
        endRadius: Union[SymbolicConstant, float],
    ) -> Path:
        """This method creates a Path object.

        .. note:: 
            This function can be accessed by::

                session.Path

        Parameters
        ----------
        name
            A String specifying the repository key.
        type
            A SymbolicConstant specifying the type of path being created. Possible values are
            NODE_LIST, POINT_LIST, EDGE_LIST, CIRCUMFERENTIAL, and RADIAL.
        expression
            A sequence specifying the nodes or points that make up the path. The definition of the
            path expression depends on the **type** argument.
            
            - If **type** = NODE_LIST, **expression** must be a sequence of sequences. Each inner sequence
              contains two items, the first item is a String specifying the name of a part instance,
              and the second item can be either a sequence of Ints or a sequence of Strings, each
              specifying a range of Ints.
            - If **type** = POINT_LIST, **expression** must be a sequence of tuples of three Floats,
              specifying the coordinates of each point.
            - If **type** = EDGE_LIST, **expression** must be a sequence of sequences. Each inner sequence
              contains two items, the first item is a String specifying the name of the part instance,
              and the second item is a sequence of tuples of four Ints that uniquely identify an
              element edge. The four Ints are:
              1. The element label.
              2. The element face index (one-based).
              3. The face edge index (one-based).
              4. The edge direction. A positive number specifies that the edge direction runs from the
              edge start node to the edge end node. A negative number specifies the opposite.
            - When **type** = CIRCUMFERENTIAL or RADIAL, **expression** must be a sequence of three tuples
              of three Floats, specifying the coordinates of the points used to define a coordinate
              system.
        circleDefinition
            A SymbolicConstant specifying the method in which the circle is being defined. This
            argument is valid only when **type** = CIRCUMFERENTIAL or RADIAL. Possible values are
            ORIGIN_AXIS and POINT_ARC.When the value is ORIGIN_AXIS, the first two points in
            **expression** are points on the rotational axis and the third point lies on the x-axis.
            When the value is POINT_ARC, the three points in **expression** are points lying on the
            arc of the circle.
        numSegments
            An Int specifying the number of equal segments in the path. This argument is valid only
            when **type** = CIRCUMFERENTIAL or RADIAL.
        startAngle
            A Float specifying the start angle of the circumferential path. This argument is valid
            only when **type** = CIRCUMFERENTIAL.
        endAngle
            A Float specifying the end angle of the circumferential path. This argument is valid
            only when **type** = CIRCUMFERENTIAL.
        radius
            The SymbolicConstant CIRCLE_RADIUS or a Float specifying the radius of the
            circumferential path. This argument is valid only when **type** = CIRCUMFERENTIAL.
        radialAngle
            A Float specifying the angle between the radial path and the **X**-axis of the specified
            coordinate system. This argument is valid only when **type** = RADIAL.
        startRadius
            The SymbolicConstant CIRCLE_RADIUS or a Float specifying the start radius of the radial
            path. This argument is valid only when **type** = RADIAL.
        endRadius
            The SymbolicConstant CIRCLE_RADIUS or a Float specifying the end radius of the radial
            path. This argument is valid only when **type** = RADIAL.

        Returns
        -------
        Path
            A :py:class:`~abaqus.PathAndProbe.Path.Path` object.

        Raises
        ------
        ModelError, ErrorUnsupportedNodeData, ErrorUnsupportedPointData, ErrorIncorrectPathData,
        KeyError, ErrorEmptyPathName, ErrorPathNotFound, and ErrorNoOdbPathDisplay.
        ValueError
            When **type** = CIRCUMFERENTIAL or RADIAL, the three points specified in
            **expression** are collinear.
        """
        self.paths[name] = path = Path(
            name,
            type,
            expression,
            circleDefinition,
            numSegments,
            startAngle,
            endAngle,
            radius,
            radialAngle,
            startRadius,
            endRadius,
        )
        return path

    @abaqus_method_doc
    def Spectrum(self, name: str, colors: tuple) -> Spectrum:
        """This method creates a Spectrum object and places it in the spectrums repository.

        .. note:: 
            This function can be accessed by::

                session.Spectrum

        Parameters
        ----------
        name
            A string name for the spectrum.
        colors
            A sequence of strings indicating the colors of the spectrum.

        Returns
        -------
        Spectrum
            A :py:class:`~abaqus.PathAndProbe.Spectrum.Spectrum` object.
        """
        self.spectrums[name] = spectrum = Spectrum(name, colors)
        return spectrum

    @abaqus_method_doc
    def Stream(
        self,
        name: str,
        numPointsOnRake: str,
        pointA: tuple = (),
        pointB: tuple = (),
        path: str = "",
    ) -> Stream:
        """This method creates aStream object and places it in the streams repository.

        .. note:: 
            This function can be accessed by::

                session.Stream

        Parameters
        ----------
        name
            A string name for the stream.
        numPointsOnRake
            An integer specifying the number of points along the rake.
        pointA
            A tuple of 3 floats specifying the starting point of the rake. Alternatively, a string
            representation of the node selected in the viewport.
        pointB
            A tuple of 3 floats specifying the end point of the rake. Alternatively, a string
            representation of the node selected in the viewport.
        path
            APath object that specifies the rake.

        Returns
        -------
        Stream
            A :py:class:`~abaqus.PathAndProbe.Stream.Stream` object.
        """
        self.streams[name] = stream = Stream(
            name, numPointsOnRake, pointA, pointB, path
        )
        return stream



from typing import Optional

from abqpy.decorators import abaqus_class_doc
from .LoadState import LoadState
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class SurfacePoreFluidState(LoadState):
    """The SurfacePoreFluidState object stores the propagating data for a SurfacePoreFluid
    object in a step. One instance of this object is created internally by the
    SurfacePoreFluid object for each step. The instance is also deleted internally by the
    SurfacePoreFluid object.
    The SurfacePoreFluidState object has no constructor or methods.
    The SurfacePoreFluidState object is derived from the LoadState object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - DSFLOW
    """

    #: A Float specifying the surface pore fluid flow magnitude.
    magnitude: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the surface pore fluid flow
    #: magnitude. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    magnitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class SmoothingAssignment:
    """The SmoothingAssignment object stores the surface smoothing assignment definition for
    surfaces in ContactExp and ContactStd objects. The SmoothingAssignment object has no
    constructor or members.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name].smoothingAssignments

        The corresponding analysis keywords are:

        - SURFACE PROPERTY ASSIGNMENT
    """

    @abaqus_method_doc
    def changeValuesInStep(self, stepName: str, index: int, value: SymbolicConstant):
        """This method allows modification of surface smoothing assignments already defined on
        surfaces in a given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the surface smoothing assignments are
            to be modified.
        index
            An Int specifying the position of the surface smoothing assignment whose value is to be
            modified.
        value
            A tuple specifying the value of the surface smoothing assignments for the surface whose
            index is referenced. Each tuple contains one entry:A SymbolicConstant specifying the
            surface smoothing value to be used for the surface. Possible values of the
            SymbolicConstant are NONE, REVOLUTION, SPHERICAL, and TOROIDAL.
        """
        ...

    @abaqus_method_doc
    def appendInStep(self, stepName: str, assignments: SymbolicConstant):
        """This method allows addition of surface smoothing assignments to new surfaces in a given
        step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which new surface smoothing assignments are
            to be defined.
        assignments
            A sequence of tuples specifying the surface smoothing assignments. Each tuple contains
            two entries:
            
            - A region object specifying the surface to which the smoothing is assigned.
            - A SymbolicConstant specifying the surface smoothing value to be used for the surface.
              Possible values of the SymbolicConstant are NONE, REVOLUTION, SPHERICAL, and TOROIDAL.
        """
        ...

    @abaqus_method_doc
    def delete(self, indices: tuple):
        """The delete method allows you to delete existing surface smoothing assignments from
        ContactExp and ContactStd objects.

        Parameters
        ----------
        indices
            A sequence of Ints specifying the index of each surface smoothing assignment to delete.
        """
        ...



from typing import overload, Dict, List, Optional, Tuple

# prevent circular imports
from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .AcisFile import AcisFile
from .PartFeature import PartFeature
from ..BasicGeometry.Cell import Cell
from ..BasicGeometry.CellArray import CellArray
from ..BasicGeometry.Edge import Edge
from ..BasicGeometry.EdgeArray import EdgeArray
from ..BasicGeometry.Face import Face
from ..BasicGeometry.FaceArray import FaceArray
from ..BasicGeometry.IgnoredEdgeArray import IgnoredEdgeArray
from ..BasicGeometry.IgnoredVertexArray import IgnoredVertexArray
from ..BasicGeometry.ReferencePoints import ReferencePoints
from ..BasicGeometry.VertexArray import VertexArray
from ..Datum.Datum import Datum
from ..Datum.DatumCsys import DatumCsys
from ..EngineeringFeature.EngineeringFeature import EngineeringFeature
from ..Mesh.MeshEdge import MeshEdge
from ..Mesh.MeshEdgeArray import MeshEdgeArray
from ..Mesh.MeshElement import MeshElement
from ..Mesh.MeshElementArray import MeshElementArray
from ..Mesh.MeshFace import MeshFace
from ..Mesh.MeshFaceArray import MeshFaceArray
from ..Mesh.MeshNode import MeshNode
from ..Mesh.MeshNodeArray import MeshNodeArray
from ..Property.CompositeLayup import CompositeLayup
from ..Property.MaterialOrientationArray import MaterialOrientationArray
from ..Property.SectionAssignmentArray import SectionAssignmentArray
from ..Region.Set import Set
from ..Region.Skin import Skin
from ..Region.Stringer import Stringer
from ..Region.Surface import Surface
from ..Sketcher.ConstrainedSketch import ConstrainedSketch
from ..UtilityAndView.abaqusConstants import (ALL_EDGES, BOUNDARY_ONLY, Boolean, FALSE, GEOMETRY,
                                              LOW, NONE, OFF, ON, SymbolicConstant, UNDEFORMED)


@abaqus_class_doc
class PartInstance:
    ...


class PartBase(PartFeature):
    """The Part object defines the physical attributes of a structure. Parts are instanced into
    the assembly and positioned before an analysis.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name]
    """

    #: A Boolean specifying the validity of the geometry of the part. The value is computed,
    #: but it can be set to ON to perform feature and mesh operations on an invalid part. There
    #: is no guarantee that such operations will work if the part was originally invalid.
    geometryValidity: Boolean = OFF

    #: An Int specifying that feature parameters have been modified but that the part has not
    #: been regenerated. Possible values are 0 and 1.
    isOutOfDate: Optional[int] = None

    #: A Float specifying when the part was last modified.
    timeStamp: Optional[float] = None

    #: A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object specifying all the vertices in the part.
    vertices: VertexArray = VertexArray([])

    #: An :py:class:`~abaqus.BasicGeometry.IgnoredVertexArray.IgnoredVertexArray` object specifying all the ignored vertices in the part.
    ignoredVertices: IgnoredVertexArray = []

    #: An :py:class:`~abaqus.BasicGeometry.EdgeArray.EdgeArray` object specifying all the edges in the part.
    edges: EdgeArray = EdgeArray([])

    #: An :py:class:`~abaqus.BasicGeometry.IgnoredEdgeArray.IgnoredEdgeArray` object specifying all the ignored edges in the part.
    ignoredEdges: IgnoredEdgeArray = []

    #: A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object specifying all the faces in the part.
    faces: FaceArray = FaceArray([])

    #: A :py:class:`~abaqus.BasicGeometry.CellArray.CellArray` object specifying all the cells in the part.
    cells: CellArray = CellArray([])

    #: A repository of Feature objects specifying all the features in the part.
    features: Dict[str, PartFeature] = {}

    #: A repository of Feature objects specifying all Feature objects in the part. The Feature
    #: objects in the featuresById repository are the same as the Feature objects in the
    #: features' repository. However, the key to the objects in the featuresById repository is
    #: an integer specifying the **ID**, whereas the key to the objects in the features
    #: repository is a string specifying the **name**.
    featuresById: Dict[str, PartFeature] = {}

    #: A repository of Datum objects specifying all the datums in the part.
    datums: List[Datum] = []

    #: A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object specifying all the elements in the part.
    elements: MeshElementArray = MeshElementArray([])

    #: A repository of MeshFace objects specifying all the element faces in the part. For a
    #: given element and a given face index within that element, the corresponding MeshFace
    #: object can be retrieved from the repository by using the key calculated as (i*8 + j),
    #: where i and j are zero-based element and face indices, respectively.
    elemFaces: Dict[str, MeshFace] = {}

    #: A :py:class:`~abaqus.Mesh.MeshFaceArray.MeshFaceArray` object specifying all the unique element faces in the part.
    elementFaces: MeshFaceArray = MeshFaceArray([])

    #: A :py:class:`~abaqus.Mesh.MeshNodeArray.MeshNodeArray` object specifying all the nodes in the part.
    nodes: MeshNodeArray = MeshNodeArray([])

    #: A :py:class:`~abaqus.Mesh.MeshNodeArray.MeshNodeArray` object specifying all the retained nodes in the substructure part.
    retainedNodes: MeshNodeArray = MeshNodeArray([])

    #: A repository of Set objects specifying for more information, see Set.
    sets: Dict[str, Set] = {}

    #: A repository of Set objects specifying the contents of the **allSets** repository is the
    #: same as the contents of the **sets** repository.
    allSets: Dict[str, Set] = {}

    #: A repository of Set objects specifying picked regions.
    allInternalSets: Dict[str, Set] = {}

    #: A repository of Surface objects specifying for more information, see Surface.
    surfaces: Dict[str, Surface] = {}

    #: A repository of Surface objects specifying the contents of the **allSurfaces** repository
    #: is the same as the contents of the **surfaces** repository.
    allSurfaces: Dict[str, Surface] = {}

    #: A repository of Surface objects specifying picked regions.
    allInternalSurfaces: Dict[str, Surface] = {}

    #: A repository of Skin objects specifying the skins created on the part.
    skins: Dict[str, Skin] = {}

    #: A repository of Stringer objects specifying the stringers created on the part.
    stringers: Dict[str, Stringer] = {}

    #: A repository of ReferencePoint objects.
    referencePoints: ReferencePoints = ReferencePoints()

    #: An :py:class:`~abaqus.EngineeringFeature.EngineeringFeature.EngineeringFeature` object.
    engineeringFeatures: EngineeringFeature = EngineeringFeature()

    #: A :py:class:`~abaqus.Property.SectionAssignmentArray.SectionAssignmentArray` object.
    sectionAssignments: SectionAssignmentArray = []

    #: A :py:class:`~abaqus.Property.MaterialOrientationArray.MaterialOrientationArray` object.
    materialOrientations: MaterialOrientationArray = []

    #: A repository of CompositeLayup objects.
    compositeLayups: Dict[str, CompositeLayup] = {}

    #: A repository of MeshEdge objects specifying all the element edges in the part. For a
    #: given element and a given edge index on a given face within that element, the
    #: corresponding MeshEdge object can be retrieved from the repository by using the key
    #: calculated as (i*32 + j*4 + k), where i, j, and k are zero-based element, face, and edge
    #: indices, respectively.
    elemEdges: Dict[str, MeshEdge] = {}

    #: A :py:class:`~abaqus.Mesh.MeshEdgeArray.MeshEdgeArray` object specifying all the unique element edges in the part.
    elementEdges: MeshEdgeArray = MeshEdgeArray([])

    @overload
    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        dimensionality: SymbolicConstant,
        type: SymbolicConstant,
        twist: Boolean = OFF,
    ):
        """This method creates a Part object and places it in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        dimensionality
            A SymbolicConstant specifying the dimensionality of the part. Possible values are
            THREE_D, TWO_D_PLANAR, and AXISYMMETRIC.
        type
            A SymbolicConstant specifying the type of the part. Possible values are DEFORMABLE_BODY,
            EULERIAN, DISCRETE_RIGID_SURFACE, and ANALYTIC_RIGID_SURFACE.
        twist
            A Boolean specifying whether to include a twist DEGREE OF FREEDOM in the part (only
            available when **dimensionality** = AXISYMMETRIC and **type** = DEFORMABLE_BODY). The default
            value is OFF.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
        """
        ...

    @overload
    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        objectToCopy: str,
        scale: float = 1,
        mirrorPlane: SymbolicConstant = NONE,
        compressFeatureList: Boolean = OFF,
        separate: Boolean = OFF,
    ):
        """This method copies a Part object and places the copy in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        objectToCopy
            A Part Object to be copied.
        scale
            A Float specifying the scaling factor to apply to the imported geometric entities during
            copy. If a scale is specified, **compressFeatureList** will be set to ON, regardless of
            whether it is specified in the command. The default value is 1.
        mirrorPlane
            A SymbolicConstant specifying how the part is to be mirrored during copy. Possible
            values are XYPLANE, XZPLANE, YZPLANE. If a mirror plane is specified,
            **compressFeatureList** will be set to ON, regardless of whether it is specified in the
            command. The default value is NONE.
        compressFeatureList
            A Boolean specifying whether to compress the feature list when copying a Part object.
            The default value is OFF. If **mirrorPlane** or **scale** is specified, this argument is
            ignored.When you compress the feature, list the resulting part will have a single
            feature. Any datums or sets in the original part will be lost.
        separate
            A Boolean specifying whether to separate disconnected regions into parts. The default
            value is OFF.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
        """
        ...

    @abaqus_method_doc
    def __init__(self, *args, **kwargs):
        ...

    def PartFromBooleanCut(
        self, name: str, instanceToBeCut: str, cuttingInstances: Tuple[PartInstance, ...]
    ):
        """This method creates a Part in the parts repository after subtracting or cutting the
        geometries of a group of part instances from that of a base part instance.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        instanceToBeCut
            A PartInstance specifying the base instance from which to cut other instances.
        cuttingInstances
            A sequence of PartInstance objects specifying the instances with which to cut the base
            instance.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
        """
        ...

    @abaqus_method_doc
    def PartFromBooleanMerge(
        self,
        name: str,
        instances: Tuple[PartInstance, ...],
        keepIntersections: Boolean = False,
        mergeNodes: SymbolicConstant = BOUNDARY_ONLY,
        nodeMergingTolerance: Optional[float] = None,
        removeDuplicateElements: Boolean = ON,
        domain: SymbolicConstant = GEOMETRY,
    ):
        """This method creates a Part in the parts repository after merging two or more part
        instances. The part instances can be either Abaqus native parts or orphan mesh parts,
        but they cannot be a combination of both.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        instances
            A sequence of PartInstance objects specifying the part instances to merge.
        keepIntersections
            A Boolean specifying whether the boundary intersections of Abaqus native part instances
            should be retained after the merge operation. The default value is False.
        mergeNodes
            A SymbolicConstant specifying whether the nodes of orphan mesh part instances should be
            retained after the merge operation. Possible values are BOUNDARY_ONLY, ALL, or NONE. The
            default value is BOUNDARY_ONLY.
        nodeMergingTolerance
            A Float specifying the maximum distance between nodes of orphan mesh part instances that
            will be merged and replaced with a single new node. The location of the new node is the
            average position of the deleted nodes. The default value is 10-6.
        removeDuplicateElements
            A Boolean specifying whether elements with the same connectivity after the merge will
            merged into a single element. The default value is ON.
        domain
            A SymbolicConstant specifying whether the part instances being merged are geometric
            instances or mesh instances. Possible values are GEOMETRY, MESH or BOTH. The default
            value is GEOMETRY.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
        """
        ...

    @abaqus_method_doc
    def PartFromExtrude2DMesh(
        self, name: str, part: "PartBase", depth: float, elementSize: float
    ):
        """This method creates a Part object by extruding an existing two-dimensional orphan mesh
        Part object in the positive **Z**-direction and places it in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        part
            A :py:class:`~abaqus.Part.Part.Part` object specifying an existing two-dimensional orphan mesh Part object.
        depth
            A Float specifying the total extrusion distance.
        elementSize
            A Float specifying an approximate element length in the extruded direction.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the specified part is not an orphan mesh part:
              Cannot extrude a geometric part.
            - If the specified part is not two-dimensional:
              Cannot extrude a 3D part.
            - If the specified part is a rigid body:
              Cannot change dimension of a rigid body.
        """
        ...

    @abaqus_method_doc
    def PartFromGeometryFile(
        self,
        name: str,
        geometryFile: AcisFile,
        dimensionality: SymbolicConstant,
        type: SymbolicConstant,
        bodyNum: int = 1,
        combine: Boolean = False,
        booleanSolids: Boolean = FALSE,
        retainBoundary: Boolean = FALSE,
        usePartNameFromFile: Boolean = OFF,
        stitchTolerance: float = 1,
        twist: Boolean = OFF,
        scale: float = 1,
        convertToAnalytical: int = 0,
        convertToPrecise: int = 0,
    ):
        """This method creates a Part object and places it in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        geometryFile
            An :py:class:`~abaqus.Part.AcisFile.AcisFile` object specifying a file containing geometry.
        dimensionality
            A SymbolicConstant specifying the dimensionality of the part. Possible values are
            THREE_D, TWO_D_PLANAR, and AXISYMMETRIC.
        type
            A SymbolicConstant specifying the type of the part. Possible values are DEFORMABLE_BODY,
            EULERIAN, DISCRETE_RIGID_SURFACE, and ANALYTIC_RIGID_SURFACE.
        bodyNum
            An Int specifying the desired body to be selected from an ACIS object containing a list
            of **N** ACIS bodies. Possible values are 1 ≤ **bodyNum** ≤ **N**. The default value is 1.
        combine
            A Boolean specifying weather to create a single part by combining all the bodies in the
            ACIS object. This argument is ignored if **bodyNum** is specified. The default value is
            False.
        booleanSolids
            A Boolean specifying whether the solids should be boolean while combining all the
            bodies.The default value is FALSE.
        retainBoundary
            A Boolean specifying whether the intersecting boundaries should be retained while
            boolean the solids.The default value is FALSE.
        usePartNameFromFile
            A Boolean specifying whether the part names specified in a STEP file should be used as
            the names in the Abaqus model database. If this option is TRUE, the part names in the
            STEP file will be used; if FALSE, each imported part will be named using the text of the
            **name** argument followed by a number. This functionality is available only for import
            from STEP files; for import from all other types of files this option should be FALSE.
        stitchTolerance
            A Float indicating the maximum gap to be stitched. The value should be smaller than the
            minimum feature size and bigger than the maximum gap expected to be stitched in the
            model. Otherwise this command may remove small (sliver) edges that are smaller than the
            tolerance. The default value is 1.0
        twist
            A Boolean specifying whether to include a twist DEGREE OF FREEDOM in the part (only
            available when **dimensionality** = AXISYMMETRIC and **type** = DEFORMABLE_BODY). The default
            value is OFF.
        scale
            A Float specifying the scaling factor to apply to the imported geometric entities. The
            default value is 1.0.
        convertToAnalytical
            An Int specifying whether to convert to analytical entities. Possible values are 0 or 1.
            The default value is 0. If **convertToAnalytical** = 1, all the numerical entities, such as
            splines, are converted to analytical entities, such as arcs and lines, during the repair
            phase of the command.
        convertToPrecise
            An Int specifying whether to convert to precise geometry. Possible value are 0 or 1. The
            default value is 0. If **convertToPrecise** = 1, the application will attempt to re-evaluate
            the tolerant entities to be more precise.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the ACIS file is corrupt:
              PartError: the file is corrupt
            - If the dimensionality does not correspond to what is found in the ACIS file:
              PartError: dimensionality does not match the contents of the file
            - If the type does not correspond to what is found in the ACIS file:
              PartError: type does not match the contents of the file
        """
        ...

    @abaqus_method_doc
    def PartFromInstanceMesh(
        self,
        name: str,
        partInstances: Tuple[PartInstance, ...] = (),
        copyPartSets: Boolean = False,
        copyAssemblySets: Boolean = False,
    ):
        """This method creates a Part object containing the mesh found in the supplied PartInstance
        objects and places the new Part object in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        partInstances
            A sequence of PartInstance objects to be used in the creation of the new mesh part. If
            the **partInstances** argument is omitted, the new Part object contains the mesh of all
            the part instances in the assembly.
        copyPartSets
            A Boolean specifying whether to copy sets, surfaces, and attributes from the base part
            or parts of the specified part instances to the new part. The default is False.
        copyAssemblySets
            A Boolean specifying whether to copy assembly-level sets that reference entities of the
            specified part instances to the new part. The default is False.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the analysis type (deformable or rigid) is not consistent among the supplied part
              instances:
              The selected part instances do not have a consistent analysis type.
            - If the assembly does not contain a mesh:
              The current assembly does not contain a mesh for a mesh part.
            - If the specified part instances do not contain a mesh:
              The selected part instances do not have a mesh for a mesh part.
        """
        ...

    @abaqus_method_doc
    def PartFromMesh(self, name: str, copySets: Boolean = False):
        """This method creates a Part object containing the mesh found in the part and places the
        new Part object in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        copySets
            A Boolean specifying whether to copy sets, surfaces, and attributes to the new part. The
            default is False.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the part does not contain a mesh:
              The current part does not contain a mesh for a mesh part.
        """
        ...

    @abaqus_method_doc
    def PartFromMeshMirror(
        self, name: str, part: "PartBase", point1: tuple, point2: tuple
    ):
        """This method creates a Part object by mirroring an existing orphan mesh Part object about
        a specified plane and places it in the parts repository. The result is a union of the
        original and the mirrored copy. Contrast the PartFromMeshMirror method with the
        **mirrorPlane** argument of the Part copy constructor. The **mirrorPlane** argument creates
        only the second half of the part but does not unite the two halves.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        part
            A :py:class:`~abaqus.Part.Part.Part` object specifying an existing orphan mesh part.
        point1
            A sequence of three Floats specifying a point on the mirror plane. This point is the
            local origin in the local system of the plane.
        point2
            A sequence of three Floats specifying a point in the direction of the normal to the
            mirror plane. This point must not be coincident with **point1**.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the specified part is not an orphan mesh part:
              Cannot mirror a geometric part.
            - If the specified part is a rigid body:
              Cannot mirror a rigid body.
            - If **point1** and **point2** are coincident:
              Mirror plane director has zero length.
            - If the specified part is two-dimensional and the plane is not parallel to the
              **Z**-axis:
              Mirror plane must be parallel to Z axis for 2D parts
        """
        ...

    @abaqus_method_doc
    def PartFromNodesAndElements(
        self,
        name: str,
        dimensionality: SymbolicConstant,
        type: SymbolicConstant,
        nodes: tuple,
        elements: tuple,
        twist: Boolean = OFF,
    ):
        """This method creates a Part object from nodes and elements and places it in the parts
        repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        dimensionality
            A SymbolicConstant specifying the dimensionality of the part. Possible values are
            THREE_D, TWO_D_PLANAR, and AXISYMMETRIC.
        type
            A SymbolicConstant specifying the type of the part. Possible values are DEFORMABLE_BODY,
            EULERIAN, DISCRETE_RIGID_SURFACE, and ANALYTIC_RIGID_SURFACE.
        nodes
            A sequence of (*nodeLabels*, **nodeCoords**) specifying the nodes of the mesh.
            **nodeLabels** is a sequence of Ints specifying the node labels, and **nodeCoords** is a
            sequence of sequences of three Floats specifying the nodal coordinates.
        elements
            A sequence of sequences of(*meshType*, **elementLabels**, **elementConns**) specifying the
            elements of the mesh. **meshType** is a String specifying the element type.
            **elementlabels** is a sequence of Ints specifying the element labels. **elementConns** is a
            sequence of sequences of node labels specifying the element connectivity.
        twist
            A boolean specifying whether the part is defined with twist. This option has meaning
            only when **dimensionality** = AXISYMMETRIC. Possible values are ON and OFF. The default
            value is OFF.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
        """
        ...

    @abaqus_method_doc
    def PartFromOdb(
        self,
        name: str,
        odb: str,
        fileName: str = "",
        instance: str = "",
        elementSet: str = "",
        shape: SymbolicConstant = UNDEFORMED,
        step: Optional[int] = None,
        frame: Optional[int] = None,
        twist: Boolean = OFF,
    ):
        """This method creates an orphan mesh Part object by reading an output database. The new
        part is placed in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        odb
            An output database object.
        fileName
            A String specifying the name of the output database file from which to create the part.
            The default value is an empty string.
        instance
            A String specifying the part instance in the output database from which to create the
            part. If no instance name is specified, Abaqus creates an orphan mesh part from the
            first part instance in the output database.
        elementSet
            A String specifying an element set defined on the output database. Only elements from
            this set will be imported. The default is to import all element sets.
        shape
            A SymbolicConstant specifying the configuration state. Possible values are UNDEFORMED
            and DEFORMED. The default value is UNDEFORMED.
        step
            An Int specifying the step number for reading deformed coordinates. 0≤step≤N−10≤step≤N-1
            where NN is the number of available steps. The default value is the last available step.
            You should specify the **step** argument only when **shape** = DEFORMED.
        frame
            An Int specifying the frame number for reading deformed coordinates.
            0≤frame≤N−10≤frame≤N-1 where NN is the number of available frames. The default value is
            the last available frame. You should specify the **frame** argument only when
            **shape** = DEFORMED.
        twist
            A Boolean specifying whether to include a twist DEGREE OF FREEDOM in the part (only
            available when **dimensionality** = AXISYMMETRIC and **type** = DEFORMABLE_BODY). The default
            value is OFF.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the output database contains elements of more than one dimensionality or type:
              File contains both axisymmetric and nonaxisymmetric elements.File contains both 2D and
              3D elements.File contains both rigid and deformable elements.
            - If more than one part is found on the output database:
              PartError: importing of more than one part is not currently supported
            - If the output database does not contain any valid results for the specified step:
              Error. File does not contain any valid frames.
            - If the specified step and frame do not contain any displacements:
              Error. Specified frame does not contain nodal displacements.
            - If the specified element set is not found on the output database:
              Error. Specified element set is not defined in the ODB.
            - If the step number is invalid:
              OdiError: Invalid step index: i. Available step indices: 0 - j.
            - If the frame number is invalid:
              OdiError: Invalid frame index: i. Available frame indices: 0 - j.
        """
        ...

    @abaqus_method_doc
    def PartFromSection3DMeshByPlane(
        self, name: str, part: "PartBase", point1: float, point2: float, point3: tuple
    ):
        """This method creates a Part object by cutting an existing three-dimensional orphan mesh
        Part object by a plane and places it in the parts repository. This method is valid only
        for orphan mesh parts composed of 8-node brick elements.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        part
            A :py:class:`~abaqus.Part.Part.Part` object specifying an existing three-dimensional orphan mesh part.
        point1
            A Sequence of three Floats specifying a point on the cutting plane. This point is the
            local origin in the local system of the plane.
        point2
            A Sequence of three Floats specifying a point in the direction of the normal to the
            cutting plane. This point must not be coincident with **point1**.
        point3
            A sequence of three Floats specifying the direction of the local 1-axis in the local
            system of the plane. This point must not project onto **point1**.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the specified part is not an orphan mesh part:
              Cannot reduce dimension of a geometric part.
            - If the specified part is not three-dimensional:
              Cannot reduce dimension of a 2D part.
            - If the specified part is a rigid body:
              Cannot change dimension of a rigid body.
            - If **point1** and **point2** are coincident:
              Cutting plane director has zero length.
            - If **point3** projects onto **point1**:
              Local axis point projects to origin.
            - If no elements are cut by the specified plane:
              Cannot reduce part dimension.
        """
        ...

    @abaqus_method_doc
    def PartFromSubstructure(self, name: str, substructureFile: str, odbFile: str):
        """This method creates a substructure Part object by reading a substructure sim file and
        places it in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        substructureFile
            A substructure sim file.
        odbFile
            The output database file corresponding to the substructure sim file.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the specified part is not a substructure:
              File specified does not contain a substructure.
            - If the specified part already exists:
              A part with the same name already exists.
            - If the substructure cannot be imported:
              The output database is missing nodes and elements.Nested substructures are not
              supported.The substructure sim file was generated using a version that is different from
              the current version.
        """
        ...

    @abaqus_method_doc
    def Part2DGeomFrom2DMesh(
        self,
        name: str,
        part: "PartBase",
        featureAngle: float,
        splineCurvatureLimit: float = 90,
        twist: Boolean = OFF,
    ):
        """This method creates a geometric Part object from the outline of an existing
        two-dimensional orphan mesh Part object and places it in the parts repository. If the
        Part2DGeomFrom2DMesh method cannot create a valid two-dimensional shell section from the
        two-dimensional mesh, the method fails and creates an empty geometry part with a failed
        base shell feature.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        part
            A :py:class:`~abaqus.Part.Part.Part` object specifying an existing two-dimensional orphan mesh Part object.
        featureAngle
            A Float specifying the angle (in degrees) between line segments that triggers a break in
            the geometry.
        splineCurvatureLimit
            A Float specifying the traversal angle in degrees of the spline that triggers a break in
            the geometry. The default value is 90.
        twist
            A Boolean specifying whether to include a twist DEGREE OF FREEDOM in the part (only
            available when **dimensionality** = AXISYMMETRIC and **type** = DEFORMABLE_BODY). The default
            value is OFF.

        Returns
        -------
        part: Part
            A :py:class:`~abaqus.Part.Part.Part` object
            
            - If the specified part is not an orphan mesh part:
              Specified part must be an orphan mesh.
            - If the Part2DGeomFrom2DMesh method cannot create a valid two-dimensional shell section from the two-dimensional mesh:
              Planar shell feature failed
            - If the specified part is not two-dimensional:
              Cannot create a geometry from a 3D part.
            - If the specified part is a rigid body:
              Cannot create a geometry from a rigid body.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Part object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def addGeomToSketch(self, sketch: ConstrainedSketch):
        """This method converts a part into a sketch by projecting all of the edges of the part
        onto the X-Y plane of the sketch. You can use addGeomToSketch with a part of any
        modeling space.

        Parameters
        ----------
        sketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object.
        """
        ...

    @abaqus_method_doc
    def assignThickness(
        self,
        faces: Tuple[Face, ...],
        thickness: Optional[float] = None,
        topFaces: Tuple[Face, ...] = (),
        bottomFaces: Tuple[Face, ...] = (),
    ):
        """This method assigns thickness data to shell faces. The thickness can be used while
        assigning shell and membrane sections to faces.

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the regions where thickness will be applied.
        thickness
            A Float specifying the thickness along the given **faces** . Either **thickness**,
            **topFaces**, or **bottomFaces** must be specified.
        topFaces
            A sequence of Face objects whose distance to **faces** argument is used to calculate the
            thickness along the **faces**. The combination of **topFaces** and **bottomFaces** determines
            the thickness and the offset of the elements. If **bottomFaces** is not specified then the
            thickness is twice the distance to the **topFaces**. This argument will be ignored if
            **thickness** is specified. Either **thickness**, **topFaces**, or **bottomFaces** must be
            specified.
        bottomFaces
            A sequence of Face objects whose distance to **faces** is used to calculate the thickness
            along the **faces**. The combination of **topFaces** and **bottomFaces** determines the
            thickness and the offset of the elements. If **topFaces** is not specified then the
            thickness is twice the distance to the **bottomFaces**. This argument will be ignored if
            **thickness** is specified. Either **thickness**, **topFaces**, or **bottomFaces** must be
            specified.
        """
        ...

    @abaqus_method_doc
    def backup(self):
        """This method makes a backup copy of the features in the part. Use the restore method to
        retrieve the part's features from the backup.
        """
        ...

    @abaqus_method_doc
    def checkGeometry(
        self,
        detailed: Boolean = OFF,
        reportFacetErrors: Boolean = OFF,
        level: Optional[int] = None,
    ):
        """This method checks the validity of the geometry of the part and prints a count of all
        topological entities on the part (faces, edges, vertices, etc.).

        Parameters
        ----------
        detailed
            A Boolean specifying whether detailed output will be printed to the replay file. The
            default value is OFF.
        reportFacetErrors
            A Boolean specifying whether faces are checked for proper facetting. The default value
            is OFF.
        level
            An Int specifying which level of checking is performed. Values can range from 20 to 70,
            with higher values reporting less and less important errors. The default value is 20,
            which reports all critical errors. When the default value is used, the stored validity
            status is updated to agree with the result of this check.
        """
        ...

    @abaqus_method_doc
    def clearGeometryCache(self):
        """This method clears the geometry cache. Clearing the geometry cache reduces the amount of
        memory being used to cache part features.
        """
        ...

    @abaqus_method_doc
    def deleteAllFeatures(self):
        """This method deletes all the features in the part."""
        ...

    @abaqus_method_doc
    def deleteFeatures(self, featureNames: tuple):
        """This method deletes the given features.

        Parameters
        ----------
        featureNames
            A sequence of Strings specifying the feature names that will be deleted from the part.
        """
        ...

    @abaqus_method_doc
    def getAngle(
        self, plane1: str, plane2: str, line1: str, line2: str, commonVertex: str = ""
    ):
        """This method returns the angle between the specified entities.

        Parameters
        ----------
        plane1
            A Face, MeshFace, or a Datum object specifying the first plane. The Datum object must
            represent a datum plane. The **plane1** and **line1** arguments are mutually exclusive. One
            of them must be specified.
        plane2
            A Face, MeshFace, or a Datum object specifying the second plane. The Datum object must
            represent a datum plane. The **plane2** and **line2** arguments are mutually exclusive. One
            of them must be specified.
        line1
            An Edge, MeshEdge, or a Datum object specifying the first curve. The Datum object must
            represent a datum axis. The **plane1** and **line1** arguments are mutually exclusive. One
            of them must be specified.
        line2
            An Edge, MeshEdge, or a Datum object specifying the second curve. The Datum object must
            represent a datum axis. The **plane2** and **line2** arguments are mutually exclusive. One
            of them must be specified.
        commonVertex
            If the two selected Edge objects have more than one vertex in common, this ConstrainedSketchVertex object
            specifies the vertex at which to evaluate the angle.

        Returns
        -------
        angle: float
            A Float specifying the angle between the specified entities. If you provide a plane as
            an argument, Abaqus/CAE computes the angle using the normal to the plane.
        """
        ...

    @abaqus_method_doc
    def getArea(self, faces: Tuple[Face, ...], relativeAccuracy: float = 0):
        """This method returns the total surface area of a given face or group of faces.

        Parameters
        ----------
        faces
            A sequence of Face objects whose area the method will calculate.
        relativeAccuracy
            A Float specifying that the area computation should stop when the specified relative
            accuracy has been achieved. The default value is 0.000001 (0.0001%).

        Returns
        -------
        area: float
            A Float specifying the sum of the calculated areas of the given faces.
        """
        ...

    @abaqus_method_doc
    def getAssociatedCADPaths(self):
        """This method returns the paths to the associated CAD part and root file. These are only
        available if the part was imported from one of the supported CAD softwares using the
        Associative Import capability. The root file can be the assembly file or the part file,
        depending on what which one was imported.

        Returns
        -------`
        paths: tuple
            A sequence containing the path to the associated CAD part and assembly file
        """
        ...

    @abaqus_method_doc
    def getCADParameters(self):
        """This method returns the names and values of the CAD parameters associated with the part.
        These are only available if the part was imported from one of the supported CAD
        softwares using the Associative Import capability, and if the parameter names defined in
        that CAD software are prefixed with the string ABQ.

        Returns
        -------
        paras: dict
            A dictionary object representing a map of the name of the parameter and its associated
            value.
        """
        ...

    @abaqus_method_doc
    def getCentroid(
        self, faces: Tuple[Face, ...], cells: Tuple[Face, ...], relativeAccuracy: float = 0
    ):
        """Location of the centroid of a given face/cell or group of faces/cells
        
        Parameters
        ----------
        faces
            A sequence of Face objects whose centroid the method will calculate. The arguments
            **faces** and **cells** are mutually exclusive.
        cells
            A sequence of Face objects whose centroid the method will calculate. The arguments
            **faces** and **cells** are mutually exclusive.
        relativeAccuracy
            A Float specifying that the centroid computation should stop when the specified relative
            accuracy has been achieved. The default value is 0.000001 (0.0001%).

        Returns
        -------
        centroid: Tuple[float, ...]
            A sequence of Floats specifying the **X**-, **Y**-, and **Z**-coordinates of the centroid.
            Depending on the arguments provided, this method returns the following:
        
            - The location of the centroid of a given face or group of faces.
            - The location of the centroid of a given cell or group of cells.
        """
        ...

    @abaqus_method_doc
    def getCoordinates(self, entity: str, csys: DatumCsys):
        """This method returns the coordinates of specified point.

        Parameters
        ----------
        entity
            A ConstrainedSketchVertex, Datum point, MeshNode, or ReferencePoint specifying the entity to query.
        csys
            A DatumCsys object specifying the desired coordinate system of the returned
            coordinates. By default, coordinates are given in the global coordinate system.

            .. versionadded:: 2022
                The `csys` argument was added.

        Returns
        -------
            A tuple of 3 Floats representing the coordinates of the specified point.
        """
        ...

    @abaqus_method_doc
    def getCurvature(self, edges: Tuple[Edge, ...], samplePoints: int = 100):
        """This method returns the maximum curvature of a given edge or group of edges. For an arc,
        the curvature is constant over the entire edge, and equal to the inverse of the radius.
        For a straight line, the curvature is constant and equal to 0. For a spline edge, the
        curvature varies over a range, and this methods computes the maximum.

        Parameters
        ----------
        edges
            A sequence of Edge objects whose curvature the method will calculate.
        samplePoints
            An Int specifying the number of points along each edge at which the curvature will be
            computed. The higher the number of sample points, the better the accuracy of the
            computation. The default value is 100.

        Returns
        -------
        curvature: float
            A Float specifying the maximum curvature.
        """
        ...

    @abaqus_method_doc
    def getDistance(self, entity1: str, entity2: str):
        """Depending on the arguments provided, this method returns one of the following:
        
        - The distance between two points.
        - The minimum distance between a point and an edge.
        - The minimum distance between two edges.

        Parameters
        ----------
        entity1
            A ConstrainedSketchVertex, Datum point, MeshNode, or Edge specifying the first entity from which to
            measure.
        entity2
            A ConstrainedSketchVertex, Datum point, MeshNode, or Edge specifying the second entity to which to
            measure.

        Returns
        -------
        distance: float
            A Float specifying the distance between **entity1** and **entity2**.
        """
        ...

    @abaqus_method_doc
    def getLength(self, edges: Tuple[Edge, ...]):
        """This method returns the length of a given edge or group of edges.

        Parameters
        ----------
        edges
            A sequence of Edge objects whose total length the method will calculate.

        Returns
        -------
        length: float
            A Float specifying the total length
        """
        ...

    @abaqus_method_doc
    def getPerimeter(self, faces: Tuple[Face, ...]):
        """This method returns the total perimeter of a given face or group of faces. All faces
        need to be on the same part. If the specified faces have shared edges, these edges are
        excluded from the computation, thus providing the length of the outer perimeter of the
        specified faces.

        Parameters
        ----------
        faces
            A sequence of Face objects whose perimeter the method will calculate.

        Returns
        -------
        perimeter: float
            A Float specifying the perimeter
        """
        ...

    @abaqus_method_doc
    def getVolume(self, cells: Tuple[Cell, ...], relativeAccuracy: float = 0):
        """This method returns the volume area of a given cell or group of cells.

        Parameters
        ----------
        cells
            A sequence of Cell objects whose volume the method will calculate.
        relativeAccuracy
            A Float specifying the relative accuracy of the computation. The default value is
            0.000001 (0.0001%).

        Returns
        -------
        volume: float
            A Float specifying the sum of the areas of the given faces
        """
        ...

    @abaqus_method_doc
    def getMassProperties(
        self,
        regions: str = "",
        relativeAccuracy: SymbolicConstant = LOW,
        useMesh: Boolean = False,
        specifyDensity: Boolean = False,
        density: str = "",
        specifyThickness: Boolean = False,
        thickness: str = "",
        miAboutCenterOfMass: Boolean = True,
        miAboutPoint: tuple = (),
    ):
        """This method returns the mass properties of a part or region. Only beams, trusses,
        shells, solids, point, nonstructural mass, and rotary inertia elements are supported.

        Parameters
        ----------
        regions
            A MeshElementArray, CellArray, FaceArray, or EdgeArray specifying the regions whose mass
            properties are to be queried. The whole part is queried by default.
        relativeAccuracy
            A SymbolicConstant specifying the relative accuracy for geometry computation. Possible
            values are LOW, MEDIUM and HIGH. The default value is LOW.
        useMesh
            A Boolean specifying whether the mesh should be used in the computation if the geometry
            is meshed. The default value is False.
        specifyDensity
            A Boolean specifying whether a user-specified density should be used in regions with
            density errors such as undefined material density. The default value is False.
        density
            A double value specifying the user-specified density value to be used in regions with
            density errors. The user-specified density should be greater than 0.
        specifyThickness
            A Boolean specifying whether a user-specified thickness should be used in regions with
            thickness errors such as undefined thickness. The default value is False.
        thickness
            A double value specifying the user-specified thickness value to be used in regions with
            thickness errors. The user-specified thickness should be greater than 0.
        miAboutCenterOfMass
            A Boolean specifying if the moments of inertia should be evaluated about the center of
            mass. The default value is True.
        miAboutPoint
            A tuple of three floats specifying the coordinates of the point about which to evaluate
            the moment of inertia. By default if the moments of inertia are not being evaluated
            about the center of mass, they will be evaluated about the origin.

        Returns
        -------
        properties: dict
            A Dictionary object with the following items:
            **area**: None or a Float specifying the sum of the area of the specified faces. The area
            is computed only for one side for shells.
            **areaCentroid**: None or a tuple of three Floats representing the coordinates of the area
            centroid.
            **volume**: None or a Float specifying the volume of the specified regions.
            **volumeCentroid**: None or a tuple of three Floats representing the coordinates of the
            volume centroid.
            **massFromMassPerUnitSurfaceArea**: None or a Float specifying the mass due to mass per
            unit surface area.
            **mass**: None or a Float specifying the mass of the specified regions. It is the total
            mass and includes mass from quantities such as mass per unit surface area.
            **centerOfMass**: None or a tuple of three Floats representing the coordinates of the
            center of mass.
            **momentOfInertia**: None or a tuple of six Floats representing the moments of inertia
            about the center of mass or about the point specified.
            **warnings**: A tuple of SymbolicConstants representing the problems encountered while
            computing the mass properties. Possible SymbolicConstants are:
            UNSUPPORTED_ENTITIES: Some unsupported entities exist in the specified region. The mass
            properties are computed only for beams, trusses, shells, solids, point and
            non-structural mass elements and rotary inertia elements. The mass properties are not
            computed for axisymmetric elements, springs, connectors, gaskets or any other elements.
            MISSING_THICKNESS: For some regions, the section definitions are missing thickness
            values.
            ZERO_THICKNESS: For some regions, the section definitions have a zero thickness value.
            VARIABLE_THICKNESS: The nodal thickness or field thickness specified for some regions
            has been ignored.
            NON_APPLICABLE_THICKNESS: For some regions, the thickness value is not applicable to the
            corresponding sections specified on the regions.
            MISSING_DENSITY: For some regions, the section definitions are missing material density
            values.
            MISSING_MATERIAL_DEFINITION: For some regions, the material definition is missing.
            ZERO_DENSITY: For some regions, the section definitions have a zero material density
            value.
            UNSUPPORTED_DENSITY: For some regions, either a negative material density or a
            temperature dependent density has been specified, or the material value is missing for
            one or more plies in the composite section.
            SHELL_OFFSETS: For shells, this method does not account for any offsets specified.
            MISSING_SECTION_DEFINITION: For some regions, the section definition is missing.
            UNSUPPORTED_SECTION_DEFINITION: The section definition provided for some regions is not
            supported.
            REINFORCEMENTS: This method does not account for any reinforcements specified on the
            model.
            SMEARED_PROPERTIES: For regions with composite section assignments, the density is
            smeared across the thickness. The volume centroid and center of mass computations for a
            composite shell use a lumped mass approach where the volume and mass is assumed to be
            lumped in the plane of the shell. As a result of these approximations the volume
            centroid, center of mass and moments of inertia may be slightly inaccurate for regions
            with composite section assignments.
            UNSUPPORTED_NON_STRUCTURAL_MASS_ENTITIES: This method does not account for any
            non-structural mass on wires.
            INCORRECT_MOMENT_OF_INERTIA: For geometry regions with non-structural mass per volume,
            the non-structural mass is assumed to be a point mass at the centroid of the regions.
            Thus, the moments of inertia may be inaccurate as the distribution of the non-structural
            mass is not accounted for. Use the mesh for accurately computing the moments of inertia.
            MISSING_BEAM_ORIENTATIONS: For some regions with beam section assignments, the beam
            section orientations are missing.
            UNSUPPORTED_BEAM_PROFILES: This method supports the Box, Pipe, Circular, Rectangular,
            Hexagonal, Trapezoidal, I, L, T, Arbitrary, and Tapered beam profiles. Any other beam
            profile is not supported.
            TAPERED_BEAM_MI: Moment of inertia calculations for tapered beams are not accurate.
            SUBSTRUCTURE_INCORRECT_PROPERTIES: The user assigned density and thickness is not
            considered for substructures.
            UNSUPPORTED_NON_STRUCTURAL_MASS_PROPORTIONAL: Non-structural mass with Mass Proportional
            distribution is not supported. Results are computed using Volume Proportional
            distribution.
        """
        ...

    @abaqus_method_doc
    def getFeatureFaces(self, name: str):
        """This method returns a sequence of Face objects that are created by the given feature.

        Parameters
        ----------
        name
            A string specifying the feature name.

        Returns
        -------
        faces: Tuple[Face, ...]
            Sequence of Face objects.

        Raises
        ------
        Error : Incorrect feature name
            An exception occurs if a feature with the given name does not exist.
        """
        ...

    @abaqus_method_doc
    def getFeatureEdges(self, name: str):
        """This method returns a sequence of Edge objects that are created by the given feature.

        Parameters
        ----------
        name
            A string specifying the feature name.

        Returns
        -------
        edges: Tuple[Edge, ...]
            Sequence of Edge objects.

        Raises
        ------
        Error: Incorrect feature name
            An exception occurs if a feature with the given name does not exist.
        """
        ...

    @abaqus_method_doc
    def getFeatureCells(self, name: str):
        """This method returns a sequence of Cell objects that are created by the given feature.

        Parameters
        ----------
        name
            A string specifying the feature name.

        Returns
        -------
        cells: Tuple[Cell, ...]
            Sequence of Cell objects.

        Raises
        ------
        Error : Incorrect feature name
            An exception occurs if a feature with the given name does not exist.
        """
        ...

    @abaqus_method_doc
    def getFeatureVertices(self, name: str):
        """This method returns a sequence of ConstrainedSketchVertex objects that are created by the given feature.

        Parameters
        ----------
        name
            A string specifying the feature name.

        Returns
        -------
        vertices: Tuple[ConstrainedSketchVertex, ...]
            Sequence of ConstrainedSketchVertex objects.

        Raises
        ------
        Error: Incorrect feature name
            An exception occurs if a feature with the given name does not exist.
        """
        ...

    @abaqus_method_doc
    def isAlignedWithSketch(self):
        """This method checks if the normal of an analytical rigid surface part is aligned with
        that of its sketch.

        Returns
        -------
        Boolean
            A Boolean value of True if the part is aligned with the sketch and False if it is not
            aligned.

        Raises
        ------
        AbaqusException: Can only be used with analytical rigid parts
            If the part is not an analytical rigid part.
        """
        ...

    @abaqus_method_doc
    def printAssignedSections(self):
        """This method prints information on each section that has been assigned to a region of the
        part.
        """
        ...

    @abaqus_method_doc
    def projectEdgesOntoSketch(
        self, sketch: str, edges: tuple, constrainToBackground: Boolean = True
    ):
        """This method projects the selected edges of a part onto the specified ConstrainedSketch
        object. The edges appear as sketch geometry after projection. If the plane of projection
        is not parallel to the specified edge, the resultant sketch geometry may be of a
        different type. For example, a circular edge can be projected as an ellipse or a line
        depending on the angle of the plane of projection. By default, the projected edge will
        be constrained to the background geometry. You can remove this constraint by setting
        **constrainToBackground** to False.

        Parameters
        ----------
        sketch
            The ConstrainedSketch object on which the edges are projected.
        edges
            A sequence of candidate edges to be projected onto the sketch.
        constrainToBackground
            A Boolean that determines whether the projected edges need to constrained to the
            background geometry. The default is True.
        """
        ...

    @abaqus_method_doc
    def projectReferencesOntoSketch(
        self,
        sketch: str,
        filter: SymbolicConstant = ALL_EDGES,
        upToFeature: Optional[PartFeature] = None,
        edges: tuple = (),
        vertices: tuple = (),
    ):
        """This method projects the vertices of specified edges, and datum points from the part
        onto the specified ConstrainedSketch object. The vertices and datum points appear on the
        sketch as reference geometry.

        Parameters
        ----------
        sketch
            The ConstrainedSketch object on which the edges, vertices, and datum points are
            projected.
        filter
            A SymbolicConstant specifying how to limit the amount of projection. Possible values are
            ALL_EDGES and COPLANAR_EDGES. If **filter** = COPLANAR_EDGES, edges that are coplanar to the
            sketching plane are the only candidates for projection. The default value is ALL_EDGES.
        upToFeature
            A :py:class:`~abaqus.Feature.Feature.Feature` object specifying a marker in the feature-based history of the part.
            Abaqus/CAE projects onto the sketch only the part entities that were created before the
            feature specified by this marker. By default, part entities in features created before
            the sketch you are editing are candidates for projection.
        edges
            A sequence of candidate edges whose vertices need to be projected onto the sketch. By
            default, all edges specified by the **filter** argument are candidates for projection.
        vertices
            A sequence of candidate vertices to be projected onto the sketch. By default, all
            vertices are candidates for projection.
        """
        ...

    @abaqus_method_doc
    def queryAttributes(self, printResults: Boolean = OFF):
        """This method prints the following information about a part:
            - the name, modeling space, and analysis type; and
            - whether twist is included (only available when the modeling space is axisymmetric and
        the analysis type is deformable); and
            - the number of vertices, edges, faces and cells if applicable.

        Parameters
        ----------
        printResults
            A Boolean which specifies whether the above information is to be printed. The default
            value is True

        Returns
        -------
        attributes: dict
            A Dictionary object with string keys and integer values which returns the above
            information with the keys being numVertices, numEdges, numFaces, numCells,
            numWiredEdges, numShellFaces and numSolidFaces.
        """
        ...

    @abaqus_method_doc
    def queryCachedStates(self):
        """This method displays the position of geometric states relative to the sequence of
        features in the part cache. The output is displayed in the message area.
        """
        ...

    @abaqus_method_doc
    def queryGeometry(self, relativeAccuracy: float = 0, printResults: Boolean = True):
        """This method prints the following information about a part:
            - the name, modeling space, and analysis type;
            - whether twist is included (only available when the modeling space is axisymmetric and
        the analysis type is deformable);
            - a 3D point representing the minimum of the part's bounding box;
            - a 3D point representing the maximum of the part's bounding box;
            - a 3D point representing the part's centroid (only on 3D solid parts); and
            - the volume (only on 3D solid parts).

        Parameters
        ----------
        relativeAccuracy
            A Float specifying that the property computations should stop when the specified
            relative accuracy has been achieved. The default value is 0.000001 (0.0001%).
        printResults
            A Boolean which specifies whether the above information is to be printed. The default
            value is True.

        Returns
        -------
        geometry: dict
            A Dictionary object with string keys, which returns the above information with the keys
            being name, space, type, volume, centroid, category and boundingBox.
        """
        ...

    @abaqus_method_doc
    def queryRegionsMissingSections(self):
        """This method returns all regions in the part that do not have a section assignment but
        require one for analysis.

        Returns
        -------
        region: Region
            A :py:class:`~abaqus.Region.Region.Region` object, or None
        """
        ...

    @abaqus_method_doc
    def queryDisjointPlyRegions(self):
        """This method provides a list of all composite plys in the current part which have
        disjoint regions.
        """
        ...

    @abaqus_method_doc
    def regenerate(self):
        """This method regenerates a part. When you modify features, it may be convenient to
        postpone regeneration until you make all your changes, since regeneration can be time
        consuming.
        """
        ...

    @abaqus_method_doc
    def regenerationWarnings(self):
        """This method prints any regeneration warnings associated with the features."""
        ...

    @abaqus_method_doc
    def removeInvalidGeometry(self):
        """Removes all invalid entities from the part, leaving a valid part. This is not recorded
        as a feature in the feature list, therefore it should be used on parts that have a
        single feature (such as an imported part).
        Note:This may remove valid entities that are connected to invalid ones. You can identify
        invalid entities using the query toolset before using this command.
        """
        ...

    @abaqus_method_doc
    def restore(self):
        """This method restores the parameters of all features in the assembly to the value they
        had before a failed regeneration. Use the restore method after a failed regeneration,
        followed by a regenerate command.
        """
        ...

    @abaqus_method_doc
    def resumeAllFeatures(self):
        """This method resumes all the suppressed features in the part."""
        ...

    @abaqus_method_doc
    def resumeFeatures(self, featureNames: tuple):
        """This method resumes the specified suppressed features in the part.

        Parameters
        ----------
        featureNames
            A tuple of names of features which are to be resumed.
        """
        ...

    @abaqus_method_doc
    def resumeLastSetFeatures(self):
        """This method resumes the last set of features to be suppressed in the part."""
        ...

    @abaqus_method_doc
    def saveGeometryCache(self):
        """This method caches the current geometry. Caching the current geometry improves
        regeneration performance.
        """
        ...

    @abaqus_method_doc
    def setAssociatedCADPaths(self, partFile: str = "", rootFile: str = ""):
        """This method sets the paths to the associated CAD part and root file. This method is only
        available if the part was imported from one of the supported CAD softwares using the
        Associative Import capability. The root file can be the assembly file or the part file,
        depending on the one that was imported. This method can be used to specify the new paths
        when the CAD data is moved to a different directory.

        Parameters
        ----------
        partFile
            A String specifying the name of the associated CAD part file.
        rootFile
            A String specifying the name of the root associated CAD file. This can be the same as
            the part file or can be the assembly file, depending on the one that was imported.
        """
        ...

    @abaqus_method_doc
    def suppressFeatures(self, featureNames: tuple):
        """This method suppresses the given features.

        Parameters
        ----------
        featureNames
            A tuple of names of features which are to be suppressed in the part.
        """
        ...

    @abaqus_method_doc
    def writeAcisFile(self, fileName: str, version: Optional[float] = None):
        """This method exports the geometry of the part to a named file in ACIS format.

        Parameters
        ----------
        fileName
            A String specifying the name of the file to which to write. The file name's extension is
            used to determine whether a part or assembly is written. Use the file extension .asat
            for the assembly format.

            .. versionchanged:: 2018
                Add description for thr file name's extension.
        version
            A Float specifying the ACIS version. For example, the Float 12.0 corresponds to ACIS
            Version 12.0. The default value is the current version of ACIS.

        Raises
        ------
        Cannot export orphan mesh parts to ACIS
            If the part is an orphan mesh part.
        """
        ...

    @abaqus_method_doc
    def writeCADParameters(
        self, paramFile: str, modifiedParams: tuple = (), updatePaths: str = ""
    ):
        """This method writes the parameters that were imported from the CAD system to a parameter
        file.

        Parameters
        ----------
        paramFile
            A String specifying the parameter file name.
        modifiedParams
            A tuple of tuples each containing the part name, the parameter name, and the modified
            parameter value. Default is an empty tuple.
        updatePaths
            A Bool specifying whether to update the path of the CAD model file specified in the
            **parameterFile** to the current directory, if the CAD model is present in the current
            directory.
        """
        ...

    @abaqus_method_doc
    def writeIgesFile(self, fileName: str, flavor: SymbolicConstant):
        """This method exports the geometry of the part to a named file in IGES format.

        Parameters
        ----------
        fileName
            A String specifying the name of the file to which to write.
        flavor
            A SymbolicConstant specifying a particular flavor of IGES. Possible values are STANDARD,
            AUTOCAD, SOLIDWORKS, JAMA, and MSBO.

        Raises
        ------
        Cannot export orphan mesh parts to IGES
            If the part is an orphan mesh part.
        """
        ...

    @abaqus_method_doc
    def writeStepFile(self, fileName: str):
        """This method exports the geometry of the part to a named file in STEP format.

        Parameters
        ----------
        fileName
            A String specifying the name of the file to which to write.

        Raises
        ------
        Parterror: Cannot export orphan mesh parts to STEP
            If the part contains no geometry.
        """
        ...

    @abaqus_method_doc
    def writeVdaFile(self, fileName: str):
        """This method exports the geometry of the part to a named file in VDA-FS format.

        Parameters
        ----------
        fileName
            A String specifying the name of the file to which to write.

        Raises
        ------
        Cannot export orphan mesh parts to VDA-FS
            If the part is an orphan mesh part.
        """
        ...

    @abaqus_method_doc
    def copyMeshPattern(
        self,
        elements: Tuple[MeshElement, ...],
        faces: Tuple[Face, ...],
        elemFaces: Tuple[MeshFace, ...],
        targetFace: MeshFace,
        nodes: Tuple[MeshNode, ...],
        coordinates: tuple,
    ):
        """This method copies a mesh pattern from a source region consisting of a set of shell
        elements or element faces onto a target face, mapping nodes and elements in a one-one
        correspondence between source and target.

        Parameters
        ----------
        elements
            A sequence of MeshElement objects or a Set object containing elements and specifying the
            source region.
        faces
            A sequence of Face objects that have associated with shell elements or element faces and
            specifying the source region.
        elemFaces
            A sequence of MeshFace objects specifying the source region.
        targetFace
            A :py:class:`~abaqus.Mesh.MeshFace.MeshFace` object specifying the target region.
        nodes
            A sequence of MeshNode objects or a Set object containing nodes on the boundary of
            source region which are to be positioned to the boundary of target face.
        coordinates
            A sequence of three-dimensional coordinate tuples specifying the coordinates for each of
            the given nodes. When specified, the number of coordinate tuples must match the number
            of given nodes, and be ordered to correspond to the given nodes in *ascending order*
            according to index. These coordinates are positions of the nodes of a mesh that will be
            the target face corresponding to nodes provided.
        """
        ...

    @abaqus_method_doc
    def smoothNodes(self, nodes: Tuple[MeshNode, ...]):
        """This method smooths the given nodes of a native mesh, moving them locally to a more
        optimal location that improves the quality of the mesh

        Parameters
        ----------
        nodes
            A sequence of MeshNode objects or a Set object containing nodes.
        """
        ...

    @abaqus_method_doc
    def Lock(self):
        """This method locks the part. Locking the part prevents any further changes to the part
        that can trigger regeneration of the part.
        """
        ...

    @abaqus_method_doc
    def Unlock(self):
        """This method unlocks the part. Unlocking the part allows it to be regenerated after any
        modifications to the part.
        """
        ...

    @abaqus_method_doc
    def LockForUpgrade(self):
        """This method locks the part for upgrade. Locking the part prevents any further changes to
        the part that can trigger regeneration of the part. When the part is unlocked, all the
        parts are upgraded and regenrated.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .GeometricRestriction import GeometricRestriction
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import AUTO, OVERHANG_REGION, SymbolicConstant


@abaqus_class_doc
class TopologyOverhangControl(GeometricRestriction):
    """The TopologyOverhangControl object defines a topology overhang control geometric
    restriction.
    The TopologyOverhangControl object is derived from the GeometricRestriction object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]

    .. versionadded:: 2019
        The `TopologyOverhangControl` class was added.
    """

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        pullDirection: tuple,
        region: Region,
        csys: Optional[int] = None,
        draftAngle: float = 45,
        overhangCheckRegion: SymbolicConstant = OVERHANG_REGION,
        pointRegion: Region = Region(),
        radius: Optional[float] = None,
        technique: SymbolicConstant = AUTO,
    ):
        """This method creates a TopologyOverhangControl object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].TopologyOverhangControl

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        pullDirection
            A VertexArray object of length 2 specifying the overhang control print direction.
            Instead of through a ConstrainedSketchVertex, each point can be specified through a tuple of coordinates.
        region
            A Region object specifying the region to which the geometric restriction is applied.
        csys
            None or a DatumCsys object specifying the local coordinate system of the
            *pullDirection*. If **csys** = None, the global coordinate system is used. When this member
            is queried, it returns an Int indicating the identifier of the DatumCsys. The default
            value is None.
        draftAngle
            A Float specifying the overhang angle. The default value is 45.0.
        overhangCheckRegion
            The SymbolicConstant OVERHANG_REGION or a Region object specifying the overhang check
            region. If the value is OVERHANG_REGION, the value of **region** is used as both the
            overhang control region and the overhang check region. The default value is
            OVERHANG_REGION.
        pointRegion
            A Region object specifying the point on a plane perpendicular to the *pullDirection*
            that is used to specify the base plane when **technique** is POINT.
        radius
            A Float specifying the radius to define the size of the cones that are used in the
            internal check for the overhang criteria.
        technique
            A SymbolicConstant specifying the overhang control technique used to define the base
            plane. Possible values are AUTO, POINT, and NONE. The default value is AUTO.

        Returns
        -------
            A TopologyOverhangControl object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        csys: Optional[int] = None,
        draftAngle: float = 45,
        overhangCheckRegion: SymbolicConstant = OVERHANG_REGION,
        pointRegion: Region = Region(),
        radius: Optional[float] = None,
        technique: SymbolicConstant = AUTO,
    ):
        """This method modifies the TopologyOverhangControl object.

        Parameters
        ----------
        csys
            None or a DatumCsys object specifying the local coordinate system of the
            *pullDirection*. If **csys** = None, the global coordinate system is used. When this member
            is queried, it returns an Int indicating the identifier of the DatumCsys. The default
            value is None.
        draftAngle
            A Float specifying the overhang angle. The default value is 45.0.
        overhangCheckRegion
            The SymbolicConstant OVERHANG_REGION or a Region object specifying the overhang check
            region. If the value is OVERHANG_REGION, the value of **region** is used as both the
            overhang control region and the overhang check region. The default value is
            OVERHANG_REGION.
        pointRegion
            A Region object specifying the point on a plane perpendicular to the *pullDirection*
            that is used to specify the base plane when **technique** is POINT.
        radius
            A Float specifying the radius to define the size of the cones that are used in the
            internal check for the overhang criteria.
        technique
            A SymbolicConstant specifying the overhang control technique used to define the base
            plane. Possible values are AUTO, POINT, and NONE. The default value is AUTO.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .DerivedComponent import DerivedComponent
from ..UtilityAndView.abaqusConstants import ABS, COMPONENT_NUMBER, POSITIVE, SymbolicConstant


@abaqus_class_doc
class ConnectorPotential:
    """The ConnectorPotential object is used to define a restricted set of mathematical
    functions to represent yield or limiting surfaces in the space spanned by connector
    available components. It can be used only in conjunction with ConnectorDamage,
    ConnectorFriction, and ConnectorPlasticity objects. Because the ConnectorDamage object
    contains two separate ConnectorPotential repositories (one for damage initiation and one
    for damage evolution), there are two ConnectorPotential constructors associated with
    that behavior—IniPotential and EvoPotential.

    .. note:: 
        This object can be accessed by::

            import section
            mdb.models[name].sections[name].behaviorOptions[i].connectorPotentials[i]
            mdb.models[name].sections[name].behaviorOptions[i].evolutionPotentials[i]
            mdb.models[name].sections[name].behaviorOptions[i].initiationPotentials[i]
            import odbSection
            session.odbs[name].sections[name].behaviorOptions[i].connectorPotentials[i]
            session.odbs[name].sections[name].behaviorOptions[i].evolutionPotentials[i]
            session.odbs[name].sections[name].behaviorOptions[i].initiationPotentials[i]

        The corresponding analysis keywords are:

        - CONNECTOR POTENTIAL
    """

    #: A :py:class:`~abaqus.Connector.ConnectorBehaviorOption.DerivedComponent` object specifying the DerivedComponent used in the contribution. This
    #: argument is applicable only if **componentStyle** = DERIVED_COMPONENT.
    derivedComponent: DerivedComponent = DerivedComponent()

    #: A SymbolicConstant specifying whether a component number or the name of the
    #: DerivedComponent object will be used in the contribution. Possible values are
    #: COMPONENT_NUMBER and DERIVED_COMPONENT. The default value is COMPONENT_NUMBER.
    componentStyle: SymbolicConstant = COMPONENT_NUMBER

    #: An Int specifying the component number used in the contribution. This argument is
    #: applicable only if **componentStyle** = COMPONENT_NUMBER. Possible values are 1 ≤
    #: **componentNumber** ≤ 6. Only available components can be specified. The default value is
    #: 0.
    componentNumber: int = 0

    #: A SymbolicConstant specifying the sign of the contribution. Possible values are POSITIVE
    #: and NEGATIVE. The default value is POSITIVE.
    sign: SymbolicConstant = POSITIVE

    #: A Float specifying the scaling factor for the contribution. The default value is 1.0.
    scaleFactor: float = 1

    #: A Float specifying the positive exponent for the contribution. The default value is
    #: 2.0.This argument is ignored if the potential operator of the invoking behavior option
    #: is set to MAXIMUM.
    positiveExponent: float = 2

    #: A Float specifying the shift factor for the contribution. The default value is 0.0.
    shiftFactor: float = 0

    #: A SymbolicConstant specifying the H function of the contribution: either absolute value,
    #: Macauley bracket, or the identity function. Possible values are ABS, MACAULEY, and
    #: IDENTITY. The default value is ABS.The value of IDENTITY can be used only if
    #: **positiveExponent** = 1.0 and the potential exponent of the invoking behavior option is
    #: also 1.0 (i.e., the potential operator of the invoking behavior option must be SUM).
    hFunction: SymbolicConstant = ABS

    @abaqus_method_doc
    def __init__(
        self,
        componentStyle: SymbolicConstant = COMPONENT_NUMBER,
        componentNumber: int = 0,
        sign: SymbolicConstant = POSITIVE,
        scaleFactor: float = 1,
        positiveExponent: float = 2,
        shiftFactor: float = 0,
        hFunction: SymbolicConstant = ABS,
    ):
        """This method creates a connector potential object to be used in conjunction with an
        allowable connector behavior option.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].sections[name].behaviorOptions[i].ConnectorPotential
                session.odbs[name].sections[name].behaviorOptions[i].ConnectorPotential
            
        Parameters
        ----------
        componentStyle
            A SymbolicConstant specifying whether a component number or the name of the 
            DerivedComponent object will be used in the contribution. Possible values are 
            COMPONENT_NUMBER and DERIVED_COMPONENT. The default value is COMPONENT_NUMBER. 
        componentNumber
            An Int specifying the component number used in the contribution. This argument is 
            applicable only if **componentStyle** = COMPONENT_NUMBER. Possible values are 1 ≤ 
            **componentNumber** ≤ 6. Only available components can be specified. The default value is 
            0. 
        sign
            A SymbolicConstant specifying the sign of the contribution. Possible values are POSITIVE 
            and NEGATIVE. The default value is POSITIVE. 
        scaleFactor
            A Float specifying the scaling factor for the contribution. The default value is 1.0. 
        positiveExponent
            A Float specifying the positive exponent for the contribution. The default value is 
            2.0.This argument is ignored if the potential operator of the invoking behavior option 
            is set to MAXIMUM. 
        shiftFactor
            A Float specifying the shift factor for the contribution. The default value is 0.0. 
        hFunction
            A SymbolicConstant specifying the H function of the contribution: either absolute value, 
            Macauley bracket, or the identity function. Possible values are ABS, MACAULEY, and 
            IDENTITY. The default value is ABS.The value of IDENTITY can be used only if 
            **positiveExponent** = 1.0 and the potential exponent of the invoking behavior option is 
            also 1.0 (i.e., the potential operator of the invoking behavior option must be SUM). 

        Returns
        -------
        ConnectorPotential
            A :py:class:`~abaqus.Connector.ConnectorPotential.ConnectorPotential` object. 

        Raises
        ------
        ValueError and TextError 
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ConnectorPotential object.

        Raises
        ------
        ValueError
        """
        ...

    @abaqus_method_doc
    def DerivedComponent(self) -> DerivedComponent:
        """This method creates a DerivedComponent object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].sections[name].behaviorOptions[i].ConnectorPotential
                session.odbs[name].sections[name].behaviorOptions[i].ConnectorPotential

        Returns
        -------
        DerivedComponent
            A :py:class:`~abaqus.Connector.ConnectorBehaviorOption.DerivedComponent` object.

        Raises
        ------
        ValueError and TextError
        """
        self.derivedComponent = derivedComponent = DerivedComponent()
        return derivedComponent
